##########################################################################
#
# Processor specific code

# CPU = "6800"
# Description = "Motorola 6800 8-bit microprocessor."
# DataWidth = 8  # 8-bit data
# AddressWidth = 16  # 16-bit addresses

# Maximum length of an instruction (for formatting purposes)
maxLength = 3

# Leadin bytes for multibyte instructions
leadInBytes = []

# Addressing mode table
# List of addressing modes and corresponding format strings for operands.
addressModeTable = {
"implied"     : "",
"immediate"   : "#${0:02X}",
"immediatex"  : "#${0:02X}{1:02X}",
"direct"      : "${0:02X}",
"indexed"     : "${0:02X},x",
"extended"    : "${0:02X}{1:02X}",
"relative"    : "${0:04X}",
}

# Op Code Table
# Key is numeric opcode (possibly multiple bytes)
# Value is a list:
#   # bytes
#   mnemonic
#   addressing mode
#   flags (e.g. pcr)
opcodeTable = {
0x01 : [ 1, "nop",  "implied"        ],
0x06 : [ 1, "tap",  "implied"        ],
0x07 : [ 1, "tpa",  "implied"        ],
0x08 : [ 1, "inx",  "implied"        ],
0x09 : [ 1, "dex",  "implied"        ],
0x0a : [ 1, "clv",  "implied"        ],
0x0b : [ 1, "sev",  "implied"        ],
0x0c : [ 1, "clc",  "implied"        ],
0x0d : [ 1, "sec",  "implied"        ],
0x0e : [ 1, "cli",  "implied"        ],
0x0f : [ 1, "sei",  "implied"        ],

0x10 : [ 1, "sba",  "implied"        ],
0x11 : [ 1, "cba",  "implied"        ],
0x14 : [ 1, "nba",  "implied"        ],
0x16 : [ 1, "tab",  "implied"        ],
0x17 : [ 1, "tba",  "implied"        ],
0x19 : [ 1, "daa",  "implied"        ],
0x1b : [ 1, "aba",  "implied"        ],

0x20 : [ 2, "bra",  "relative", pcr  ],
0x22 : [ 2, "bhi",  "relative", pcr  ],
0x23 : [ 2, "bls",  "relative", pcr  ],
0x24 : [ 2, "bcc",  "relative", pcr  ],
0x25 : [ 2, "bcs",  "relative", pcr  ],
0x26 : [ 2, "bne",  "relative", pcr  ],
0x27 : [ 2, "beq",  "relative", pcr  ],
0x28 : [ 2, "bvc",  "relative", pcr  ],
0x29 : [ 2, "bvs",  "relative", pcr  ],
0x2a : [ 2, "bpl",  "relative", pcr  ],
0x2b : [ 2, "bmi",  "relative", pcr  ],
0x2c : [ 2, "bge",  "relative", pcr  ],
0x2d : [ 2, "blt",  "relative", pcr  ],
0x2e : [ 2, "bgt",  "relative", pcr  ],
0x2f : [ 2, "ble",  "relative", pcr  ],

0x30 : [ 1, "tsx",  "implied"        ],
0x31 : [ 1, "ins",  "implied"        ],
0x32 : [ 1, "pula", "implied"        ],
0x33 : [ 1, "pulb", "implied"        ],
0x34 : [ 1, "des",  "implied"        ],
0x35 : [ 1, "txs",  "implied"        ],
0x36 : [ 1, "psha", "implied"        ],
0x37 : [ 1, "pshb", "implied"        ],
0x39 : [ 1, "rts",  "implied"        ],
0x3b : [ 1, "rti",  "implied"        ],
0x3e : [ 1, "wai",  "implied"        ],
0x3f : [ 1, "swi",  "implied"        ],

0x40 : [ 1, "nega", "implied"        ],
0x43 : [ 1, "com",  "implied"        ],
0x44 : [ 1, "lsra", "implied"        ],
0x46 : [ 1, "rora", "implied"        ],
0x47 : [ 1, "asra", "implied"        ],
0x48 : [ 1, "asla", "implied"        ],
0x49 : [ 1, "rola", "implied"        ],
0x4a : [ 1, "deca", "implied"        ],
0x4c : [ 1, "inca", "implied"        ],
0x4d : [ 1, "tsta", "implied"        ],
0x4f : [ 1, "clra", "implied"        ],

0x50 : [ 1, "neg",  "implied"        ],
0x53 : [ 1, "comb", "implied"        ],
0x54 : [ 1, "lsrb", "implied"        ],
0x56 : [ 1, "rorb", "implied"        ],
0x57 : [ 1, "asrb", "implied"        ],
0x58 : [ 1, "aslb", "implied"        ],
0x59 : [ 1, "rolb", "implied"        ],
0x5a : [ 1, "decb", "implied"        ],
0x5c : [ 1, "incb", "implied"        ],
0x5d : [ 1, "tstb", "implied"        ],
0x5f : [ 1, "clrb", "implied"        ],

0x60 : [ 2, "neg",  "indexed"        ],
0x63 : [ 2, "com",  "indexed"        ],
0x64 : [ 2, "lsr",  "indexed"        ],
0x66 : [ 2, "ror",  "indexed"        ],
0x67 : [ 2, "asr",  "indexed"        ],
0x68 : [ 2, "asl",  "indexed"        ],
0x69 : [ 2, "rol",  "indexed"        ],
0x6a : [ 2, "dec",  "indexed"        ],
0x6c : [ 2, "inc",  "indexed"        ],
0x6d : [ 2, "tst",  "indexed"        ],
0x6e : [ 2, "jmp",  "indexed"        ],
0x6f : [ 2, "clr",  "indexed"        ],

0x70 : [ 3, "neg",  "extended"       ],
0x73 : [ 3, "com",  "extended"       ],
0x74 : [ 3, "lsr",  "extended"       ],
0x76 : [ 3, "ror",  "extended"       ],
0x77 : [ 3, "asr",  "extended"       ],
0x78 : [ 3, "asl",  "extended"       ],
0x79 : [ 3, "rol",  "extended"       ],
0x7a : [ 3, "dec",  "extended"       ],
0x7c : [ 3, "inc",  "extended"       ],
0x7d : [ 3, "tst",  "extended"       ],
0x7e : [ 3, "jmp",  "extended"       ],
0x7f : [ 3, "clr",  "extended"       ],

0x80 : [ 2, "sub",  "immediate"      ],
0x81 : [ 2, "cmp",  "immediate"      ],
0x82 : [ 2, "sbc",  "immediate"      ],
0x84 : [ 2, "and",  "immediate"      ],
0x85 : [ 2, "bit",  "immediate"      ],
0x86 : [ 2, "lda",  "immediate"      ],
0x88 : [ 2, "eor",  "immediate"      ],
0x89 : [ 2, "adc",  "immediate"      ],
0x8a : [ 2, "ora",  "immediate"      ],
0x8b : [ 2, "add",  "immediate"      ],
0x8c : [ 3, "cpx",  "immediatex"     ],
0x8d : [ 2, "bsr",  "relative"       ],
0x8e : [ 3, "lds",  "immediatex"     ],

0x90 : [ 2, "sub",  "direct"         ],
0x91 : [ 2, "cmp",  "direct"         ],
0x92 : [ 2, "sbc",  "direct"         ],
0x94 : [ 2, "and",  "direct"         ],
0x95 : [ 2, "bit",  "direct"         ],
0x96 : [ 2, "lda",  "direct"         ],
0x97 : [ 2, "sta",  "direct"         ],
0x98 : [ 2, "eor",  "direct"         ],
0x99 : [ 2, "adc",  "direct"         ],
0x9a : [ 2, "ora",  "direct"         ],
0x9b : [ 2, "add",  "direct"         ],
0x9c : [ 2, "cpx",  "direct"         ],
0x9e : [ 2, "lds",  "direct"         ],
0x9f : [ 2, "sts",  "direct"         ],

0xa0 : [ 2, "sub",  "indexed"        ],
0xa1 : [ 2, "cmp",  "indexed"        ],
0xa2 : [ 2, "sbc",  "indexed"        ],
0xa4 : [ 2, "and",  "indexed"        ],
0xa5 : [ 2, "bit",  "indexed"        ],
0xa6 : [ 2, "ldaa", "indexed"        ],
0xa7 : [ 2, "staa", "indexed"        ],
0xa8 : [ 2, "eora", "indexed"        ],
0xa9 : [ 2, "adca", "indexed"        ],
0xaa : [ 2, "oraa", "indexed"        ],
0xab : [ 2, "adda", "indexed"        ],
0xac : [ 2, "cpx",  "indexed"        ],
0xad : [ 2, "jsr",  "indexed"        ],
0xae : [ 2, "lds",  "indexed"        ],
0xaf : [ 2, "sts",  "indexed"        ],

0xb0 : [ 3, "suba", "extended"       ],
0xb1 : [ 3, "cmpa", "extended"       ],
0xb2 : [ 3, "sbca", "extended"       ],
0xb4 : [ 3, "anda", "extended"       ],
0xb5 : [ 3, "bita", "extended"       ],
0xb6 : [ 3, "ldaa", "extended"       ],
0xb7 : [ 3, "staa", "extended"       ],
0xb8 : [ 3, "eora", "extended"       ],
0xb9 : [ 3, "adca", "extended"       ],
0xba : [ 3, "oraa", "extended"       ],
0xbb : [ 3, "adda", "extended"       ],
0xbc : [ 3, "cpx",  "extended"       ],
0xbd : [ 3, "jsr",  "extended"       ],
0xbe : [ 3, "lds",  "extended"       ],
0xbf : [ 3, "sts",  "extended"       ],

0xc0 : [ 2, "subb", "immediate"      ],
0xc1 : [ 2, "cmpb", "immediate"      ],
0xc2 : [ 2, "sbcb", "immediate"      ],
0xc4 : [ 2, "andb", "immediate"      ],
0xc5 : [ 2, "bitb", "immediate"      ],
0xc6 : [ 2, "ldab", "immediate"      ],
0xc8 : [ 2, "eorb", "immediate"      ],
0xc9 : [ 2, "adcb", "immediate"      ],
0xca : [ 2, "orab", "immediate"      ],
0xcb : [ 2, "addb", "immediate"      ],
0xce : [ 3, "ldx",  "immediatex"     ],

0xd0 : [ 2, "subb", "direct"         ],
0xd1 : [ 2, "cmpb", "direct"         ],
0xd2 : [ 2, "sbcb", "direct"         ],
0xd4 : [ 2, "andb", "direct"         ],
0xd5 : [ 2, "bitb", "direct"         ],
0xd6 : [ 2, "ldab", "direct"         ],
0xd7 : [ 2, "stab", "direct"         ],
0xd8 : [ 2, "eorb", "direct"         ],
0xd9 : [ 2, "adcb", "direct"         ],
0xda : [ 2, "orab", "direct"         ],
0xdb : [ 2, "addb", "direct"         ],
0xde : [ 2, "ldx",  "direct"         ],
0xdf : [ 2, "stx",  "direct"         ],

0xe0 : [ 2, "subb", "indexed"        ],
0xe1 : [ 2, "cmpb", "indexed"        ],
0xe2 : [ 2, "sbcb", "indexed"        ],
0xe4 : [ 2, "andb", "indexed"        ],
0xe5 : [ 2, "bitb", "indexed"        ],
0xe6 : [ 2, "ldab", "indexed"        ],
0xe7 : [ 2, "stab", "indexed"        ],
0xe8 : [ 2, "eorb", "indexed"        ],
0xe9 : [ 2, "adcb", "indexed"        ],
0xea : [ 2, "orab", "indexed"        ],
0xeb : [ 2, "addb", "indexed"        ],
0xee : [ 2, "ldx",  "indexed"        ],
0xef : [ 2, "stx",  "indexed"        ],

0xf0 : [ 3, "subb", "extended"       ],
0xf1 : [ 3, "cmpb", "extended"       ],
0xf2 : [ 3, "sbcb", "extended"       ],
0xf4 : [ 3, "andb", "extended"       ],
0xf5 : [ 3, "bitb", "extended"       ],
0xf6 : [ 3, "ldab", "extended"       ],
0xf7 : [ 3, "stab", "extended"       ],
0xf8 : [ 3, "eorb", "extended"       ],
0xf9 : [ 3, "adcb", "extended"       ],
0xfa : [ 3, "orab", "extended"       ],
0xfb : [ 3, "addb", "extended"       ],
0xfe : [ 3, "ldx",  "extended"       ],
0xff : [ 3, "stx",  "extended"       ],

}

# End of processor specific code
##########################################################################



#------------------------------------------------------------------------------
#
#  Copyright (c) 2005, Enthought, Inc.
#  All rights reserved.
#
#  This software is provided without warranty under the terms of the BSD
#  license included in enthought/LICENSE.txt and may be redistributed only
#  under the conditions described in the aforementioned license.  The license
#  is also available online at http://www.enthought.com/licenses/BSD.txt
#
#  Thanks for using Enthought open source!
#
#  Author: David C. Morrill
#  Date:   10/21/2004
#
#------------------------------------------------------------------------------

""" Defines the various Boolean editors for the wxPython user interface toolkit.
"""

#-------------------------------------------------------------------------------
#  Imports:
#-------------------------------------------------------------------------------

import wx

# FIXME: ToolkitEditorFactory is a proxy class defined here just for backward
# compatibility. The class has been moved to the
# traitsui.editors.boolean_editor file.
from traitsui.editors.boolean_editor \
    import ToolkitEditorFactory

from editor \
    import Editor

# This needs to be imported in here for use by the editor factory for boolean
# editors (declared in traitsui). The editor factory's text_editor
# method will use the TextEditor in the ui.
from text_editor \
    import SimpleEditor as TextEditor

from constants \
    import ReadonlyColor

#-------------------------------------------------------------------------------
#  'SimpleEditor' class:
#-------------------------------------------------------------------------------

class SimpleEditor ( Editor ):
    """ Simple style of editor for Boolean values, which displays a check box.
    """
    #---------------------------------------------------------------------------
    #  Finishes initializing the editor by creating the underlying toolkit
    #  widget:
    #---------------------------------------------------------------------------

    def init ( self, parent ):
        """ Finishes initializing the editor by creating the underlying toolkit
            widget.
        """
        self.control = wx.CheckBox( parent, -1, '' )
        wx.EVT_CHECKBOX( parent, self.control.GetId(), self.update_object )
        self.set_tooltip()

    #---------------------------------------------------------------------------
    #  Handles the user clicking on the checkbox:
    #---------------------------------------------------------------------------

    def update_object ( self, event ):
        """ Handles the user clicking the checkbox.
        """
        self.value = (self.control.GetValue() != 0)

    #---------------------------------------------------------------------------
    #  Updates the editor when the object trait changes external to the editor:
    #---------------------------------------------------------------------------

    def update_editor ( self ):
        """ Updates the editor when the object trait changes externally to the
            editor.
        """
        self.control.SetValue( self.value )

#-------------------------------------------------------------------------------
#  'ReadonlyEditor' class:
#-------------------------------------------------------------------------------

class ReadonlyEditor ( Editor ):
    """ Read-only style of editor for Boolean values, which displays static text
    of either "True" or "False".
    """
    #---------------------------------------------------------------------------
    #  Finishes initializing the editor by creating the underlying toolkit
    #  widget:
    #---------------------------------------------------------------------------

    def init ( self, parent ):
        """ Finishes initializing the editor by creating the underlying toolkit
            widget.
        """
        self.control = wx.TextCtrl( parent, -1, '', style = wx.TE_READONLY )
        self.control.SetBackgroundColour( ReadonlyColor )

    #---------------------------------------------------------------------------
    #  Updates the editor when the object trait changes external to the editor:
    #
    #  (Should normally be overridden in a subclass)
    #---------------------------------------------------------------------------

    def update_editor ( self ):
        """ Updates the editor when the object trait changes externally to the
            editor.
        """
        if self.value:
            self.control.SetValue( 'True' )
        else:
            self.control.SetValue( 'False' )



""" Bitmap editor

"""
# Enthought library imports.
from pyface.api import GUI, ImageResource, ConfirmationDialog, FileDialog, \
    ImageResource, YES, OK, CANCEL
from pyface.action.api import Action, ActionItem, Separator, Group
from pyface.tasks.api import Task, TaskWindow, TaskLayout, PaneItem, IEditor, \
    IEditorAreaPane, EditorAreaPane, Editor, DockPane, HSplitter, VSplitter
from pyface.tasks.action.api import DockPaneToggleGroup, SMenuBar, \
    SMenu, SToolBar, TaskAction, TaskToggleGroup, EditorAction, SchemaAddition
from traits.api import on_trait_change, Property, Instance, Any, Event, Int

from omnivore.framework.actions import *
from omnivore.tasks.hex_edit.task import HexEditTask
from omnivore.tasks.hex_edit.actions import *
from omnivore.tasks.hex_edit.preferences import HexEditPreferences
import omnivore.arch.colors as colors
from omnivore.framework.toolbar import get_toolbar_group

from jumpman_editor import JumpmanEditor
from commands import *
import pane_layout
from actions import *


class JumpmanEditTask(HexEditTask):
    """ Jumpman level display decoder
    """

    new_file_text = "Jumpman Level"

    #### Task interface #######################################################

    id = pane_layout.task_id_with_pane_layout
    name = 'Jumpman Level Editor'
    
    preferences_helper = HexEditPreferences
    
    #### Menu events ##########################################################
    
    
    ###########################################################################
    # 'Task' interface.
    ###########################################################################

    def _default_layout_default(self):
        return pane_layout.pane_layout()

    def create_dock_panes(self):
        return pane_layout.pane_create()

    def _extra_actions_default(self):
        data_menu = self.create_menu("Menu", "Disk Image", "ParserGroup", "EmulatorGroup", "ActionGroup")
        segment_menu = self.create_menu("Menu", "Segments", "SegmentGroup")
        actions = [
            # Menubar additions
            SchemaAddition(factory=lambda: segment_menu,
                           path='MenuBar',
                           after="Edit",
                           ),
            SchemaAddition(factory=lambda: data_menu,
                           path='MenuBar',
                           after="Edit",
                           ),
            ]
        return actions

    def _tool_bars_default(self):
        toolbars = []
        toolbars.append(get_toolbar_group("%s:Modes" % self.id, JumpmanEditor.valid_mouse_modes))
        toolbars.extend(HexEditTask._tool_bars_default(self))
        return toolbars

    def pane_layout_initial_visibility(self):
        return pane_layout.pane_initially_visible()

    ###########################################################################
    # 'FrameworkTask' interface.
    ###########################################################################

    def get_editor(self, guess=None):
        """ Opens a new empty window
        """
        editor = JumpmanEditor()
        return editor

    @on_trait_change('window.application.preferences_changed_event')
    def refresh_from_new_preferences(self):
        e = self.active_editor
        if e is not None:
            prefs = self.get_preferences()

    def get_actions_Menu_Edit_UndoGroup(self):
        return [
            UndoAction(),
            RedoAction(),
            ]
    
    def get_actions_Menu_Edit_CopyPasteGroup(self):
        return [
            CutAction(),
            CopyAction(),
            PasteAction(),
            ]
    
    def get_actions_Menu_Edit_SelectGroup(self):
        return [
            SelectAllJumpmanAction(),
            SelectNoneJumpmanAction(),
            SelectInvertJumpmanAction(),
            ]
    
    def get_actions_Menu_Edit_FindGroup(self):
        return [
            FlipVerticalAction(),
            FlipHorizontalAction(),
            ]

    def get_actions_Menu_View_ViewPredefinedGroup(self):
        return []

    def get_actions_Menu_View_ViewChangeGroup(self):
        return [
            SMenu(
                Group(
                    ColorStandardAction(name="NTSC", color_standard=0),
                    ColorStandardAction(name="PAL", color_standard=1),
                    id="a0", separator=True),
                Group(
                    UseColorsAction(name="Powerup Colors", colors=colors.powerup_colors()),
                    id="a1", separator=True),
                Group(
                    AnticColorAction(),
                    id="a2", separator=True),
                id='mm4', separator=False, name="Colors"),
            ]

    ###
    @classmethod
    def can_edit(cls, document):
        return document.metadata.mime == "application/octet-stream" or document.segments
    
    @classmethod
    def get_match_score(cls, document):
        """Return a number based on how good of a match this task is to the
        incoming Document.
        
        0 = generic match
        ...
        10 = absolute match
        """
        return 0



#------------------------------------------------------------------------------
#
#  Copyright (c) 2005, Enthought, Inc.
#  All rights reserved.
#
#  This software is provided without warranty under the terms of the BSD
#  license included in enthought/LICENSE.txt and may be redistributed only
#  under the conditions described in the aforementioned license.  The license
#  is also available online at http://www.enthought.com/licenses/BSD.txt
#
#  Thanks for using Enthought open source!
#
#  Author: David C. Morrill
#  Date:   10/21/2004
#
#------------------------------------------------------------------------------

""" Defines the various font editors and the font editor factory, for the
    wxPython user interface toolkit..
"""

#-------------------------------------------------------------------------------
#  Imports:
#-------------------------------------------------------------------------------

import wx

from traits.api import Bool

from traitsui.editors.font_editor \
    import ToolkitEditorFactory as BaseToolkitEditorFactory

from editor_factory \
    import SimpleEditor as BaseSimpleEditor, \
    TextEditor as BaseTextEditor, \
    ReadonlyEditor as BaseReadonlyEditor

from editor \
    import Editor

from helper \
    import TraitsUIPanel, disconnect

#-------------------------------------------------------------------------------
#  Constants:
#-------------------------------------------------------------------------------

# Standard font point sizes
PointSizes = [
   '8',  '9', '10', '11', '12', '14', '16', '18',
  '20', '22', '24', '26', '28', '36', '48', '72'
]

# All available font styles
Styles = ['Normal', 'Slant', 'Italic']

# All available font weights
Weights = ['Normal', 'Light', 'Bold']

# All available font facenames
facenames = None

#---------------------------------------------------------------------------
#  The wxPython ToolkitEditorFactory class.
#---------------------------------------------------------------------------
## We need to add wx-specific methods to the editor factory, and so we create
## a subclass of the BaseToolkitEditorFactory.

class ToolkitEditorFactory(BaseToolkitEditorFactory):
    """ wxPython editor factory for font editors.
    """

    show_style = Bool(False)
    show_weight = Bool(False)

    #---------------------------------------------------------------------------
    #  Returns a wxFont object corresponding to a specified object's font trait:
    #---------------------------------------------------------------------------

    def to_wx_font ( self, editor ):
        """ Returns a wxFont object corresponding to a specified object's font
        trait.
        """
        font = editor.value
        return wx.Font( font.GetPointSize(), font.GetFamily(), font.GetStyle(),
                        font.GetWeight(),    font.GetUnderlined(),
                        font.GetFaceName() )

    #---------------------------------------------------------------------------
    #  Gets the application equivalent of a wxPython Font value:
    #---------------------------------------------------------------------------

    def from_wx_font ( self, font ):
        """ Gets the application equivalent of a wxPython Font value.
        """
        return font

    #---------------------------------------------------------------------------
    #  Returns the text representation of the specified object trait value:
    #---------------------------------------------------------------------------

    def str_font ( self, font ):
        """ Returns the text representation of the specified object trait value.
        """
        weight = { wx.LIGHT: ' Light',
                    wx.BOLD:  ' Bold'   }.get( font.GetWeight(), '' )
        style  = { wx.SLANT: ' Slant',
                    wx.ITALIC:' Italic' }.get( font.GetStyle(), '' )
        return '%s point %s%s%s' % (
                font.GetPointSize(), font.GetFaceName(), style, weight )

    #---------------------------------------------------------------------------
    #  Returns a list of all available font facenames:
    #---------------------------------------------------------------------------

    def all_facenames ( self ):
        """ Returns a list of all available font facenames.
        """
        global facenames

        if facenames is None:
            facenames = FontEnumerator().facenames()
            facenames.sort()
        return facenames

#-------------------------------------------------------------------------------
#  'SimpleFontEditor' class:
#-------------------------------------------------------------------------------

class SimpleFontEditor ( BaseSimpleEditor ):
    """ Simple style of font editor, which displays a text field that contains
        a text representation of the font value (using that font if possible).
        Clicking the field displays a font selection dialog box.
    """

    #---------------------------------------------------------------------------
    #  Invokes the pop-up editor for an object trait:
    #---------------------------------------------------------------------------

    def popup_editor ( self, event ):
        """ Invokes the pop-up editor for an object trait.
        """
        font_data = wx.FontData()
        font_data.SetInitialFont( self.factory.to_wx_font( self ) )
        dialog = wx.FontDialog( self.control, font_data )
        if dialog.ShowModal() == wx.ID_OK:
            self.value = self.factory.from_wx_font(
                              dialog.GetFontData().GetChosenFont() )
            self.update_editor()

        dialog.Destroy()

    #---------------------------------------------------------------------------
    #  Updates the editor when the object trait changes external to the editor:
    #---------------------------------------------------------------------------

    def update_editor ( self ):
        """ Updates the editor when the object trait changes externally to the
            editor.
        """
        super( SimpleFontEditor, self ).update_editor()
        set_font( self )

    #---------------------------------------------------------------------------
    #  Returns the text representation of a specified font value:
    #---------------------------------------------------------------------------

    def string_value ( self, font ):
        """ Returns the text representation of a specified font value.
        """
        return self.factory.str_font( font )

#-------------------------------------------------------------------------------
#  'CustomFontEditor' class:
#-------------------------------------------------------------------------------

class CustomFontEditor ( Editor ):
    """ Custom style of font editor, which displays the following:

        * A combo box containing all available type face names.
        * A combo box containing the available type sizes.
        * A combo box containing the available type styles
    """

    #---------------------------------------------------------------------------
    #  Finishes initializing the editor by creating the underlying toolkit
    #  widget:
    #---------------------------------------------------------------------------

    def init ( self, parent ):
        """ Finishes initializing the editor by creating the underlying toolkit
            widget.
        """
        # Create a panel to hold all of the buttons:
        self.control = panel = TraitsUIPanel( parent, -1 )
        sizer = wx.BoxSizer( wx.VERTICAL )

        # Add all of the font choice controls:
        sizer2    = wx.BoxSizer( wx.HORIZONTAL )
        facenames = self.factory.all_facenames()
        control   = self._facename = wx.Choice( panel, -1, wx.Point( 0, 0 ),
                                                wx.Size( -1, -1 ), facenames )

        sizer2.Add( control, 4, wx.EXPAND )
        wx.EVT_CHOICE( panel, control.GetId(), self.update_object_parts )

        control = self._point_size = wx.Choice( panel, -1, wx.Point( 0, 0 ),
                                                wx.Size( -1, -1 ), PointSizes )
        sizer2.Add( control, 1, wx.EXPAND | wx.LEFT, 3 )
        wx.EVT_CHOICE( panel, control.GetId(), self.update_object_parts )

        if self.factory.show_style:
            self._style = wx.Choice(panel, -1, wx.Point(0,0),
                                    wx.Size(-1, -1), Styles)
            sizer2.Add(self._style, 1, wx.EXPAND | wx.LEFT, 3)
            wx.EVT_CHOICE( panel, self._style.GetId(), self.update_object_parts )

        if self.factory.show_weight:
            self._weight = wx.Choice(panel, -1, wx.Point(0,0),
                                    wx.Size(-1, -1), Weights)
            sizer2.Add(self._weight, 1, wx.EXPAND | wx.LEFT, 3)
            wx.EVT_CHOICE( panel, self._weight.GetId(), self.update_object_parts )

        sizer.Add( sizer2, 0, wx.EXPAND )


        # Set-up the layout:
        panel.SetSizer( sizer )

        self.set_tooltip()

    #---------------------------------------------------------------------------
    #  Disposes of the contents of an editor:
    #---------------------------------------------------------------------------

    def dispose ( self ):
        """ Disposes of the contents of an editor.
        """
        disconnect( self._facename,   wx.EVT_CHOICE )
        disconnect( self._point_size, wx.EVT_CHOICE )
        if self.factory.show_style:
            disconnect(self._style, wx.EVT_CHOICE)
        if self.factory.show_weight:
            disconnect(self._weight, wx.EVT_CHOICE)

        super( CustomFontEditor, self ).dispose()

    #---------------------------------------------------------------------------
    #  Handles the user modifying one of the font components:
    #---------------------------------------------------------------------------

    def update_object_parts ( self, event ):
        """ Handles the user modifying one of the font components.
        """
        point_size = int( self._point_size.GetStringSelection() )
        facename   = self._facename.GetStringSelection()

        style = wx.FONTSTYLE_NORMAL
        if self.factory.show_style:
            style += self._style.GetCurrentSelection()

        weight = wx.FONTWEIGHT_NORMAL
        if self.factory.show_weight:
            weight += self._weight.GetCurrentSelection()

        font       = wx.Font( point_size, wx.DEFAULT, style, weight,
                              faceName = facename )
        self.value = self.factory.from_wx_font( font )

    #---------------------------------------------------------------------------
    #  Updates the editor when the object trait changes external to the editor:
    #---------------------------------------------------------------------------

    def update_editor ( self ):
        """ Updates the editor when the object trait changes externally to the
            editor.
        """
        font = self.factory.to_wx_font( self )

        try:
           self._facename.SetStringSelection( font.GetFaceName() )
        except:
           self._facename.SetSelection( 0 )

        try:
           self._point_size.SetStringSelection( str( font.GetPointSize() ) )
        except:
           self._point_size.SetSelection( 0 )

    #---------------------------------------------------------------------------
    #  Returns the text representation of a specified font value:
    #---------------------------------------------------------------------------

    def string_value ( self, font ):
        """ Returns the text representation of a specified font value.
        """
        return self.factory.str_font( font )


#-------------------------------------------------------------------------------
#  'TextFontEditor' class:
#-------------------------------------------------------------------------------

class TextFontEditor ( BaseTextEditor ):
    """ Text style of font editor, which displays an editable text field
        containing a text representation of the font value (using that font if
        possible).
    """

    #---------------------------------------------------------------------------
    #  Handles the user changing the contents of the edit control:
    #---------------------------------------------------------------------------

    def update_object ( self, event ):
        """ Handles the user changing the contents of the edit control.
        """
        self.value = self.control.GetValue()

    #---------------------------------------------------------------------------
    #  Updates the editor when the object trait changes external to the editor:
    #---------------------------------------------------------------------------

    def update_editor ( self ):
        """ Updates the editor when the object trait changes external to the
            editor.
        """
        super( TextFontEditor, self ).update_editor()
        set_font( self )

    #---------------------------------------------------------------------------
    #  Returns the text representation of a specified font value:
    #---------------------------------------------------------------------------

    def string_value ( self, font ):
        """ Returns the text representation of a specified font value.
        """
        return self.factory.str_font( font )

#-------------------------------------------------------------------------------
#  'ReadonlyFontEditor' class:
#-------------------------------------------------------------------------------

class ReadonlyFontEditor ( BaseReadonlyEditor ):
    """ Read-only style of font editor, which displays a read-only text field
        containing a text representation of the font value (using that font if
        possible).
    """

    #---------------------------------------------------------------------------
    #  Updates the editor when the object trait changes external to the editor:
    #---------------------------------------------------------------------------

    def update_editor ( self ):
        """ Updates the editor when the object trait changes external to the
            editor.
        """
        super( ReadonlyFontEditor, self ).update_editor()
        set_font( self )

    #---------------------------------------------------------------------------
    #  Returns the text representation of a specified font value:
    #---------------------------------------------------------------------------

    def string_value ( self, font ):
        """ Returns the text representation of a specified font value.
        """
        return self.factory.str_font( font )

#-------------------------------------------------------------------------------
#  Set the editor control's font to match a specified font:
#-------------------------------------------------------------------------------

def set_font ( editor ):
    """ Sets the editor control's font to match a specified font.
    """
    font = editor.factory.to_wx_font( editor )
    font.SetPointSize( min( 10, font.GetPointSize() ) )
    editor.control.SetFont( font )

#-------------------------------------------------------------------------------
#  'FontEnumerator' class:
#-------------------------------------------------------------------------------

class FontEnumerator ( wx.FontEnumerator ):
    """ An enumeration of fonts.
    """
    #---------------------------------------------------------------------------
    #  Returns a list of all available font facenames:
    #---------------------------------------------------------------------------

    def facenames ( self ):
        """ Returns a list of all available font facenames.
        """
        self._facenames = []
        self.EnumerateFacenames()
        return self._facenames

    #---------------------------------------------------------------------------
    #  Adds a facename to the list of facenames:
    #---------------------------------------------------------------------------

    def OnFacename ( self, facename ):
        """ Adds a facename to the list of facenames.
        """
        self._facenames.append( facename )
        return True

# Define the names SimpleEditor, CustomEditor, TextEditor and ReadonlyEditor
# which are looked up by the editor factory for the font editor.
SimpleEditor = SimpleFontEditor
CustomEditor = CustomFontEditor
TextEditor = TextFontEditor
ReadonlyEditor = ReadonlyFontEditor

### EOF #######################################################################



#!/usr/bin/env python
""" Mini-assembler that uses the formatting strings and opcode tables from
udis (the Universal Disassembler for 8-bit microprocessors by Jeff Tranter) to
perform pattern matching to determine the opcode and addressing mode.

Copyright (c) 2016 by Rob McMullen <feedback@playermissile.com>
Licensed under the Apache License 2.0
"""
from __future__ import print_function

import os
import re
from collections import defaultdict

import numpy as np

try:
    import cputables
except ImportError:
    raise RuntimeError("Generate cputables.py using cpugen.py before using the miniassembler")

from disasm import Disassembler

import logging
logging.basicConfig(level=logging.WARNING)
log = logging.getLogger(__name__)


# flags
pcr = 1
und = 2
z80bit = 4
r = 64
w = 128

class FormatSpec(object):
    """ Format specifier that combines info from the udis addressModeTable and
    the opcodeTable.
    
    During the brute force decoding, the patterns contained in the
    addressModeTable format string are checked against the arguments decoded
    from the text to be assembled. The first match is returned as the
    assembled bytes.
    """
    
    # regex to find the argument number in the format string
    fmtargre = re.compile(r'\{[0-9]:')

    def __init__(self, format, opcode, num_bytes, mode_name, flag):
        self.format = format
        self.mode_name = mode_name
        self.num_args = len(re.findall(self.fmtargre, format))
        self.length = len(format)
        if opcode is not None:
            if opcode >= 256 * 256 * 256:
                op1 = (opcode & 0xff000000) >> 24
                op2 = (opcode & 0xff0000) >> 16
                op3 = (opcode & 0xff00) >> 8
                op4 = opcode & 0xff
                self.opcode_bytes = [op1, op2, op3, op4]
            elif opcode > 255:
                op1, op2 = divmod(opcode, 256)
                self.opcode_bytes = [op1, op2]
            else:
                self.opcode_bytes = [opcode]
        else:
            self.opcode_bytes = []
        self.num_bytes = num_bytes
        self.flag = flag
    
    def __str__(self):
        return "%s: %s" % (self.mode_name, self.format)
    
    def __repr__(self):
        return "%s: %s" % (self.mode_name, self.format)
    
    def __lt__(self, other):
        # Only need to implement 'less than' for sorting to work
        return (self.length, self.format) < (other.length, other.format)
    
    def get_bytes(self, bytes=None):
        out = list(self.opcode_bytes)
        if bytes is not None:
            out.extend(bytes)
        return tuple(out)
    
    def check_exact(self, operands):
        """ Check all the format strings that have no extra arguments, like
        "NOP" or even 6502's "ASL A"
        
        """
        log.debug(" check_exact: %s -> %s" % (self.mode_name, self.format))
        if self.format == operands:
            return self.get_bytes()
    
    def check_hex_1x8(self, operands, pc, byte):
        """ Check all the format strings that have one 8-bit hex value
        
        """
        if self.num_args == 0:
            # z80 has some hardcoded values in static opcodes, like $ff = "rst
            # $38" that get parsed as an opcode and a byte value, so we ignore
            # the byte value here because of it being hardcoded.
            gen = self.format
            log.debug(" check_hex_1x8(0): " + gen + " -> " + operands)
            if gen == operands:
                return self.get_bytes()
        elif self.num_args == 1:
            gen = self.format.format(byte)
            log.debug(" check_hex_1x8(1): " + gen + " -> " + operands)
            if gen == operands:
                if self.flag & z80bit:
                    out = list(self.opcode_bytes)
                    out[2:3] = [byte]
                    return tuple(out)
                else:
                    return self.get_bytes([byte])
    
    def check_hex_1x16(self, operands, pc, low_byte, high_byte):
        """ Check all the format strings that have one 16-bit hex value,
        typically the PC-relative instructions.
        
        """
        if self.flag == pcr:
            addr = low_byte + 256 * high_byte
            offset = addr - (pc + 2)
            # offset is limited by signed 16 bit size, so find the positive or
            # negative value that clamps to that range
            if offset > 32767:
                offset -= 0x10000
            elif offset < -32768:
                offset += 0x10000
            if self.num_bytes == 2:
                log.debug(" check_hex_1x16(1): %s -> %04x (pc=%x, offset=%x)" % (self.mode_name, addr, pc, offset))
                if -128 <= offset <= 127:
                    gen = self.format.format(addr)
                    log.debug("  gen=%s operands=%s" % (gen, operands))
                    if gen == operands:
                        return self.get_bytes([offset & 0xff])  # convert to unsigned representation
            elif self.num_bytes == 3:
                log.debug(" check_hex_1x16(2): %s -> %04x (pc=%x, offset=%x)" % (self.mode_name, addr, pc, offset))
                if -32768 <= offset <= 32767:
                    gen = self.format.format(addr)
                    log.debug("  gen=%s operands=%s" % (gen, operands))
                    if gen == operands:
                        unsigned = offset & 0xffff  # convert to unsigned representation
                        high, low = divmod(unsigned, 256)
                        return self.get_bytes([low, high])
    
    def check_hex_2x8(self, operands, pc, low_byte, high_byte):
        """ Check all the format strings that have two 8-bit hex values
        
        """
        if self.num_args == 2:
            log.debug(" check_hex_2x8(2): %s -> %02x, %02x" % (self.mode_name, low_byte, high_byte))
            gen = self.format.format(low_byte, high_byte)
            log.debug("  " + gen + ":" + operands)
            if gen == operands:
                return self.get_bytes([low_byte, high_byte])
    
    def check_hex_3x8(self, operands, pc, low_byte, high_byte, bank_byte):
        """ Check all the format strings that have two 8-bit hex values
        
        """
        if self.num_args == 3:
            log.debug(" check_hex_3x8(3): %s -> %02x, %02x, %02x" % (self.mode_name, low_byte, high_byte, bank_byte))
            gen = self.format.format(low_byte, high_byte, bank_byte)
            log.debug("  " + gen + ":" + operands)
            if gen == operands:
                return self.get_bytes([low_byte, high_byte, bank_byte])


class MiniAssembler(object):
    def __init__(self, cpu_name, allow_undocumented=False):
        self.source = None
        self.setup(cpu_name, allow_undocumented)
    
    def setup(self, cpu_name, allow_undocumented):
        """ Create the opcode lookup tables that store all the possible
        addressing modes for each opcode.
        """
        cpu = cputables.processors[cpu_name]
        self.little = True  # all 8-bit processors little endian???
        
        # Create temporary format dictionary that will be used in the expanded
        # opcode lookup table
        formats = {}
        table = cpu['addressModeTable']
        for mode, fmt in table.items():
            formats[mode] = fmt.lower()
        
        # Create the opcode lookup table that holds a list of possible
        # addressing modes for each opcode. Stores the addressing mode format
        # in each list entry to eliminate a lookup to another table at the
        # cost of some extra space in this lookup table.
        d = defaultdict(list)
        table = cpu['opcodeTable']
        for opcode, optable in table.items():
            try:
                num_bytes, mnemonic, mode_name, flag = optable
            except ValueError:
                num_bytes, mnemonic, mode_name = optable
                flag = 0
            if allow_undocumented or flag & 2 == 0:
                d[mnemonic].append(FormatSpec(formats[mode_name], opcode, num_bytes, mode_name, flag))
        d[".db"].append(FormatSpec("${0:02x}", None, 1, "data_byte", 0))
        
        # Order the lookup table from smallest to largest format specifier for
        # each opcode
        self.ops = {}
        for mnemonic, modelist in d.items():
            log.debug(mnemonic)
            modelist.sort()
            log.debug(modelist)
            self.ops[mnemonic] = modelist
        self.undocumented = allow_undocumented
    
    addrre = re.compile(r'\$[0-9a-fA-F]+')
    immediatere = re.compile(r'#?\$[0-9a-fA-F]+')
    
    def parse_operands(self, opstr, operands, pc):
        try:
            format_specs = self.ops[opstr]
        except KeyError:
            raise RuntimeError("Unknown mnemonic %s" % opstr)
        log.debug("-->%s<--, -->%s<--: %s" % (opstr, operands, self.ops[opstr]))
        
        # Check if a single operand matches an address mode exactly
        for f in format_specs:
            bytes = f.check_exact(operands)
            if bytes:
                return bytes
        
        # Check for hex value
        values = re.findall(self.addrre, operands)
        if values:
            log.debug("HEX!: %s" % str(values))
            num = len(values)
            
            if num == 1:
                hexstr = values[0][1:]
                v = int(hexstr, 16)
                if len(hexstr) == 1 or len(hexstr) == 2:
                    for f in format_specs:
                        bytes = f.check_hex_1x8(operands, pc, v)
                        if bytes:
                            return bytes
                if len(hexstr) > 4:
                    vh, vl = divmod(v, 256)
                    vb, vh = divmod(vh, 256)
                    for f in format_specs:
                        bytes = f.check_hex_3x8(operands, pc, vl, vh, vb)
                        if bytes:
                            return bytes
                elif len(hexstr) > 2:
                    vh, vl = divmod(v, 256)
                    for f in format_specs:
                        bytes = f.check_hex_1x16(operands, pc, vl, vh)
                        if bytes:
                            return bytes
                        bytes = f.check_hex_2x8(operands, pc, vl, vh)
                        if bytes:
                            return bytes
            elif num == 2:
                hexstr = values[0][1:]
                vl = int(hexstr, 16)
                hexstr = values[1][1:]
                vh = int(hexstr, 16)
                for f in format_specs:
                    bytes = f.check_hex_2x8(operands, pc, vl, vh)
                    if bytes:
                        return bytes
        
        return []
    
    def asm(self, origin, text):
        text = text.lower()
        log.debug("input: %s" % text)
        if " " in text:
            opstr, operands = text.split(" ", 1)
        else:
            opstr = text
            operands = ""
        if ";" in operands:
            operands, _ = operands.split(";", 1)
        operands = operands.replace(" ", "")
        opstr = opstr.lower()
        operands = operands.strip()
        bytes = self.parse_operands(opstr, operands, origin)
        return bytes


def process(source, filename, start_pc, cpu, assemble_only=False, undoc=False, verbose=True):
    def format_hex(t):
        return "(" + ", ".join(["%02x" % s for s in t]) + ")"
    
    pc = start_pc
    miniasm = MiniAssembler(cpu, allow_undocumented=undoc)
    success = failure = 0
    if assemble_only:
        for line in source.splitlines():
            if line.startswith("0x"):
                addr, line = line.split(" ", 1)
                addr = int(addr, 16)
                line = line.strip()
            else:
                addr = pc
            try:
                bytes = miniasm.asm(addr, line)
                log.debug(bytes)
            except KeyError:
                log.debug("unrecognized", line)
                bytes = []
            print("%s: output=%s" % (line, str(bytes)))
            pc += len(bytes)
    else:
        binary = np.fromstring(source, dtype=np.uint8)
        disasm = Disassembler(cpu, allow_undocumented=undoc, hex_lower=True, mnemonic_lower=True)
        disasm.set_pc(binary, start_pc)
        for addr, disassembled_bytes, opstr, comment, flag in disasm.get_disassembly():
            assembled_bytes = miniasm.asm(addr, opstr)
            if disassembled_bytes == assembled_bytes:
                success += 1
                if verbose:
                    print("%06x %s:" % (addr, opstr), format_hex(disassembled_bytes), format_hex(assembled_bytes))
            else:
                failure += 1
                print("%06x %s:" % (addr, opstr), format_hex(disassembled_bytes), format_hex(assembled_bytes))
#            print "0x%04x %-12s ; %s   %s %s" % (addr, opstr, comment, bytes, flag)
        print("%s: %d instructions matched, %d failed" % (filename, success, failure))
    return success, failure

if __name__ == "__main__":
    import sys
    import argparse
    
    parser = argparse.ArgumentParser()
    parser.add_argument("-c", "--cpu", help="Specify CPU type (defaults to 6502)", default="6502")
    parser.add_argument("-u", "--undocumented", help="Allow undocumented opcodes", action="store_true")
    parser.add_argument("-a", "--assemble", help="Only assemble", action="store_true")
    parser.add_argument("-d", "--debug", help="Show debug information as the program runs", action="store_true")
    parser.add_argument("-v", "--verbose", help="Show processed instructions as the program runs", action="store_true")
    parser.add_argument("-x", "--hex", help="Assemble a string version of hex digits")
    parser.add_argument("-s", "--string", help="Assemble a single line opcode (implies -a)")
    parser.add_argument("-p", "--pc", help="Process counter", default="0")
    args, extra = parser.parse_known_args()
    
    if args.debug:
        log.setLevel(logging.DEBUG)
    
    try:
        start_pc = int(args.pc, 16)
    except ValueError:
        start_pc = int(args.pc)
    

    if args.string:
        source = args.string
        args.assemble = True
        process(source, args.string, start_pc, args.cpu, args.assemble, args.undocumented, args.verbose)
    elif args.hex:
        try:
            source = args.hex.decode("hex")
        except TypeError:
            print("Invalid hex digits!")
            sys.exit()
        process(source, args.string, start_pc, args.cpu, args.assemble, args.undocumented, args.verbose)
    else:
        for filename in extra:
            source = []
            with open(filename, 'rb') if filename !="-" else sys.stdin as fh:
                try:
                    chunk_length = 1024
                    while True:
                        chunk = fh.read(chunk_length)
                        source.append(chunk)
                        if len(chunk) < chunk_length:
                            break
                except KeyboardInterrupt:
                    pass
            source = "".join(source)
            if args.verbose:
                print("read %d bytes" % len(source))
            process(source, filename, start_pc, args.cpu, args.assemble, args.undocumented, args.verbose)




#------------------------------------------------------------------------------
#
#  Copyright (c) 2005, Enthought, Inc.
#  All rights reserved.
#
#  This software is provided without warranty under the terms of the BSD
#  license included in enthought/LICENSE.txt and may be redistributed only
#  under the conditions described in the aforementioned license.  The license
#  is also available online at http://www.enthought.com/licenses/BSD.txt
#
#  Thanks for using Enthought open source!
#
#  Author: David C. Morrill
#  Date:   10/21/2004
#
#------------------------------------------------------------------------------

""" Defines the compound editor and the compound editor factory for the
    wxPython user interface toolkit.
"""

#-------------------------------------------------------------------------------
#  Imports:
#-------------------------------------------------------------------------------

import wx

from traits.api \
    import Str

# FIXME: ToolkitEditorFactory is a proxy class defined here just for backward
# compatibility. The class has been moved to the
# traitsui.editors.compound_editor file.
from traitsui.editors.compound_editor \
    import ToolkitEditorFactory

from editor \
    import Editor

from helper \
    import TraitsUIPanel

#-------------------------------------------------------------------------------
#  'CompoundEditor' class:
#-------------------------------------------------------------------------------

class CompoundEditor ( Editor ):
    """ Editor for compound traits, which displays editors for each of the
    combined traits, in the appropriate style.
    """
    #---------------------------------------------------------------------------
    #  Trait definitions:
    #---------------------------------------------------------------------------

    # The kind of editor to create for each list item
    kind = Str

    #---------------------------------------------------------------------------
    #  Finishes initializing the editor by creating the underlying toolkit
    #  widget:
    #---------------------------------------------------------------------------

    def init ( self, parent ):
        """ Finishes initializing the editor by creating the underlying toolkit
            widget.
        """
        # Create a panel to hold all of the component trait editors:
        self.control = panel = TraitsUIPanel( parent, -1 )
        sizer        = wx.BoxSizer( wx.VERTICAL )

        # Add all of the component trait editors:
        self._editors = editors = []
        for factory in self.factory.editors:
            editor = getattr( factory, self.kind )( self.ui, self.object,
                                       self.name, self.description, panel )
            editor.prepare( panel )
            sizer.Add( editor.control, 1,
                       wx.TOP | wx.BOTTOM | editor.layout_style, 3 )
            editors.append( editor )

        # Set-up the layout:
        panel.SetSizerAndFit( sizer )

        self.set_tooltip()

    #---------------------------------------------------------------------------
    #  Updates the editor when the object trait changes external to the editor:
    #---------------------------------------------------------------------------

    def update_editor ( self ):
        """ Updates the editor when the object trait changes externally to the
            editor.
        """
        pass

    #---------------------------------------------------------------------------
    #  Disposes of the contents of an editor:
    #---------------------------------------------------------------------------

    def dispose ( self ):
        """ Disposes of the contents of an editor.
        """
        for editor in self._editors:
            editor.dispose()

        super( CompoundEditor, self ).dispose()

#-------------------------------------------------------------------------------
#  'SimpleEditor' class:
#-------------------------------------------------------------------------------

class SimpleEditor(CompoundEditor):

    # The kind of editor to create for each list item. This value overrides
    # the default.
    kind = 'simple_editor'

#-------------------------------------------------------------------------------
#  'CustomEditor' class:
#-------------------------------------------------------------------------------

class CustomEditor(CompoundEditor):

    # The kind of editor to create for each list item. This value overrides
    # the default.

    kind = 'custom_editor'

#-- EOF ----------------------------------------------------------------------



#-------------------------------------------------------------------------------
#
#  Copyright (c) 2007, Enthought, Inc.
#  All rights reserved.
#
#  This software is provided without warranty under the terms of the BSD
#  license included in enthought/LICENSE.txt and may be redistributed only
#  under the conditions described in the aforementioned license.  The license
#  is also available online at http://www.enthought.com/licenses/BSD.txt
#
#  Thanks for using Enthought open source!
#
#  Author: David C. Morrill
#  Date:   07/04/2007
#
#-------------------------------------------------------------------------------

""" Traits UI themed checkbox editor.
"""

#-------------------------------------------------------------------------------
#  Imports:
#-------------------------------------------------------------------------------

import wx

from traits.api \
    import Instance, Str

from traitsui.ui_traits \
    import ATheme, Image, Position, Spacing

from traitsui.wx.editor \
    import Editor

from traitsui.basic_editor_factory \
    import BasicEditorFactory

from themed_control \
    import ThemedControl

#-------------------------------------------------------------------------------
#  '_ThemedCheckboxEditor' class:
#-------------------------------------------------------------------------------

class _ThemedCheckboxEditor ( Editor ):
    """ Traits UI themed checkbox editor.
    """

    # The ThemedControl used for the checkbox:
    checkbox = Instance( ThemedControl )

    #---------------------------------------------------------------------------
    #  Finishes initializing the editor by creating the underlying toolkit
    #  widget:
    #---------------------------------------------------------------------------

    def init ( self, parent ):
        """ Finishes initializing the editor by creating the underlying toolkit
            widget.
        """
        # Create the checkbox and its control:
        item      = self.item
        factory   = self.factory
        label     = self.string_value( factory.label or item.label )
        min_size  = ( 0, 0 )
        if factory.theme is not None:
            min_size  = ( 80, 0 )

        self.checkbox = checkbox = ThemedControl( **factory.get(
            'image', 'position', 'spacing', 'theme' ) ).set(
            text       = label,
            controller = self,
            min_size   = min_size )
        self.control = checkbox.create_control( parent )

        # Set the tooltip:
        self.set_tooltip()

    #---------------------------------------------------------------------------
    #  Updates the editor when the object trait changes external to the editor:
    #---------------------------------------------------------------------------

    def update_editor ( self ):
        """ Updates the editor when the object trait changes externally to the
            editor.
        """
        if self.checkbox.state == 'hover':
            self._set_hover_theme()
        else:
            self._set_theme()

    #-- ThemedControl Event Handlers -------------------------------------------

    def normal_motion ( self, x, y, event ):
        self._set_hover_theme( 'hover' )
        self.control.CaptureMouse()

    def hover_left_down ( self, x, y, event ):
        self.control.ReleaseMouse()
        self._set_hover_theme( 'down', not self.value )

    def hover_motion ( self, x, y, event ):
        if not self.checkbox.in_control( x, y ):
            self.control.ReleaseMouse()
            self._set_theme( 'normal' )

    def down_left_up ( self, x, y, event ):
        if self.checkbox.in_control( x, y ):
            self.value = not self.value
            self.normal_motion( x, y, event )
        else:
            self._set_theme( 'normal' )

    def down_motion ( self, x, y, event ):
        if not self.checkbox.in_control( x, y ):
            self._set_theme()
        else:
            self._set_hover_theme( value = not self.value )

    #-- Private Methods --------------------------------------------------------

    def _set_theme ( self, state = None, value = None ):
        """ Sets the theme, image, offset and optional checkbox state to use for
            a specified checkbox state value.
        """
        if value is None:
            value = self.value

        factory      = self.factory
        theme, image = factory.theme, factory.image
        if value:
            theme, image = factory.on_theme, factory.on_image

        n = (1 * value) * (theme is not None)
        self.checkbox.set( offset = ( n, n ),
                           theme  = theme or factory.theme,
                           image  = image or factory.image,
                           state  = state or self.checkbox.state )

    def _set_hover_theme ( self, state = None, value = None ):
        """ Sets the theme, image, offset and optional checkbox state to use for
            a specified checkbox state value while in hover mode.
        """
        if value is None:
            value = self.value

        factory      = self.factory
        theme, image = factory.hover_off_theme, factory.hover_off_image
        if value:
            theme = factory.hover_on_theme or factory.on_theme
            image = factory.hover_on_image or factory.on_image

        n = (1 * value) * (theme is not None)
        self.checkbox.set( offset = ( n, n ),
                           theme  = theme or factory.theme,
                           image  = image or factory.image,
                           state  = state or self.checkbox.state )

#-------------------------------------------------------------------------------
#  Create the editor factory object:
#-------------------------------------------------------------------------------

# wxPython editor factory for themed checkbox editors:
class ThemedCheckboxEditor ( BasicEditorFactory ):

    # The editor class to be created:
    klass = _ThemedCheckboxEditor

    # The checkbox label:
    label = Str

    # The basic theme for the checkbox (i.e. the 'off' state):
    theme = ATheme

    # The optional 'on' state theme for the checkbox:
    on_theme = ATheme

    # The optional 'hover off' state theme for the checkbox:
    hover_off_theme = ATheme

    # The optional 'hover on' state theme for the checbox:
    hover_on_theme = ATheme

    # The optional image to display in the checkbox (i.e. the 'off' state):
    image = Image( 'cb_off' )

    # The optional 'on' state image to display in the checkbox:
    on_image = Image( 'cb_on' )

    # The optional 'hover off' state image to display in the checkbox:
    hover_off_image = Image( 'cb_hover_off' )

    # The optional 'hover on' state image to display in the checkbox:
    hover_on_image = Image( 'cb_hover_on' )

    # The position of the image relative to the text:
    position = Position

    # The amount of space between the image and the text:
    spacing = Spacing

#-------------------------------------------------------------------------------
#  Helper function for creating themed checkboxes:
#-------------------------------------------------------------------------------

def themed_checkbox_editor ( style = None, show_checkbox = True, **traits ):
    """ Simplifies creation of a ThemedCheckboxEditor by setting up the
        themes and images automatically based on the value of the *style* and
        *show_checkbox* arguments.
    """
    tce = ThemedCheckboxEditor( **traits )

    if not show_checkbox:
        tce.set( image           = None,
                 on_image        = None,
                 hover_off_image = None,
                 hover_on_image  = None )

    if isinstance( style, basestring ):
        group = style[0:1].upper()
        if (len( group ) == 0) or (group not in 'BCGJTY'):
            group = 'B'

        row      = style[1:2].upper()
        all_rows = '0123456789ABCDEFGHIJKL'
        if (len( row ) == 0) or (row not in all_rows):
            row = 'H'

        column      = style[2:3].upper()
        all_columns = '12345789AB'
        if (len( column ) == 0) or (column not in all_columns):
            column = '5'

        tce.theme = '@%s%s%s' % ( group, row, column )

        if style[-1:] == '.':
            return tce

        alt_row    = '44456349A78FFFGHEFKLIJ'[ all_rows.index( row ) ]
        alt_column = '66666CCCCC'[ all_columns.index( column ) ]

        tce.set( on_theme        = '@%s%s%s' % ( group, alt_row, column ),
                 hover_on_theme  = '@%s%s%s' % ( group, alt_row, alt_column ),
                 hover_off_theme = '@%s%s%s' % ( group, row, alt_column ) )

    return tce



from __future__ import absolute_import

from traits.testing.unittest_tools import unittest, UnittestTools

from ..gui import GUI
from ..heading_text import HeadingText
from ..split_application_window import SplitApplicationWindow


class TestSplitApplicationWindow(unittest.TestCase, UnittestTools):

    def setUp(self):
        self.gui = GUI()
        self.window = SplitApplicationWindow()

    def test_destroy(self):
        # test that destroy works even when no control
        self.window.destroy()

    def test_open_close(self):
        # test that opening and closing works as expected
        with self.assertTraitChanges(self.window, 'opening', count=1):
            with self.assertTraitChanges(self.window, 'opened', count=1):
                self.window.open()
        self.gui.process_events()
        with self.assertTraitChanges(self.window, 'closing', count=1):
            with self.assertTraitChanges(self.window, 'closed', count=1):
                self.window.close()
        self.gui.process_events()

    def test_horizontal_split(self):
        # test that horizontal split works
        self.window.direction = 'horizontal'
        with self.assertTraitChanges(self.window, 'opening', count=1):
            with self.assertTraitChanges(self.window, 'opened', count=1):
                self.window.open()
        self.gui.process_events()
        with self.assertTraitChanges(self.window, 'closing', count=1):
            with self.assertTraitChanges(self.window, 'closed', count=1):
                self.window.close()
        self.gui.process_events()

    def test_contents(self):
        # test that contents works
        self.window.lhs = HeadingText
        self.window.rhs = HeadingText
        with self.assertTraitChanges(self.window, 'opening', count=1):
            with self.assertTraitChanges(self.window, 'opened', count=1):
                self.window.open()
        self.gui.process_events()
        with self.assertTraitChanges(self.window, 'closing', count=1):
            with self.assertTraitChanges(self.window, 'closed', count=1):
                self.window.close()
        self.gui.process_events()

    def test_ratio(self):
        # test that ratio split works
        self.window.ratio = 0.25
        with self.assertTraitChanges(self.window, 'opening', count=1):
            with self.assertTraitChanges(self.window, 'opened', count=1):
                self.window.open()
        self.gui.process_events()
        with self.assertTraitChanges(self.window, 'closing', count=1):
            with self.assertTraitChanges(self.window, 'closed', count=1):
                self.window.close()
        self.gui.process_events()



#------------------------------------------------------------------------------
# Copyright (c) 2007, Riverbank Computing Limited
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in enthought/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
#------------------------------------------------------------------------------

# Standard library imports.
import os
import sys
import logging

# Enthought library imports.
from traits.etsconfig.api import ETSConfig

logger = logging.getLogger(__name__)

# This is set to the root part of the module path for the selected backend.
_toolkit_backend = None


try:
    provisional_toolkit = ETSConfig.provisional_toolkit
except AttributeError:
    from contextlib import contextmanager

    # for backward compatibility
    @contextmanager
    def provisional_toolkit(toolkit_name):
        """ Perform an operation with toolkit provisionally set

        This sets the toolkit attribute of the ETSConfig object set to the
        provided value. If the operation fails with an exception, the toolkit
        is reset to nothing.
        """
        if ETSConfig.toolkit:
            raise AttributeError("ETSConfig toolkit is already set")
        ETSConfig.toolkit = toolkit_name
        try:
            yield
        except:
            # reset the toolkit state
            ETSConfig._toolkit = ''
            raise


def _init_toolkit():
    """ Initialise the current toolkit. """

    def import_toolkit(tk):
        try:
            # Try and import the toolkit's pyface backend init module.
            be = 'pyface.ui.%s.' % tk
            __import__(be + 'init')
        except:
            raise
        return be

    # Get the toolkit.
    if ETSConfig.toolkit:
        be = import_toolkit(ETSConfig.toolkit)
    else:
        # Toolkits to check for if none is explicitly specified.
        known_toolkits = ('qt4', 'wx', 'null')

        for tk in known_toolkits:
            try:
                with provisional_toolkit(tk):
                    be = import_toolkit(tk)
                break
            except ImportError as exc:
                msg = "Could not import Pyface backend '{0}'"
                logger.info(msg.format(tk))
                if logger.getEffectiveLevel() <= logging.INFO:
                    logger.exception(exc)
        else:
            # Try to import the null toolkit but don't set the ETSConfig toolkit
            try:
                be = import_toolkit('null')
                import warnings
                msg = ("Unable to import the '{0}' backend for pyface; " +
                       "using the 'null' backend instead.")
                warnings.warn(msg.format(toolkit_name), RuntimeWarning)
            except ImportError as exc:
                logger.exception(exc)
                raise ImportError("Unable to import a pyface backend for any "
                    "of the %s toolkits" % ", ".join(known_toolkits))

    # Save the imported toolkit module.
    global _toolkit_backend
    _toolkit_backend = be


# Do this once then disappear.
_init_toolkit()
del _init_toolkit


def toolkit_object(name):
    """ Return the toolkit specific object with the given name.

    Parameters
    ----------
    name : str
        The name consists of the relative module path and the object name
        separated by a colon.
    """

    mname, oname = name.split(':')
    be_mname = _toolkit_backend + mname

    class Unimplemented(object):
        """ An unimplemented toolkit object

        This is returned if an object isn't implemented by the selected
        toolkit.  It raises an exception if it is ever instantiated.
        """

        def __init__(self, *args, **kwargs):
            raise NotImplementedError("the %s pyface backend doesn't implement %s" % (ETSConfig.toolkit, oname))

    be_obj = Unimplemented

    try:
        __import__(be_mname)

        try:
            be_obj = getattr(sys.modules[be_mname], oname)
        except AttributeError:
            pass
    except ImportError as exc:
        # is the error while trying to import be_mname or not?
        if all(part not in exc.args[0] for part in mname.split('.')):
                # something else went wrong - let the exception be raised
                raise

        # Ignore *ANY* errors unless a debug ENV variable is set.
        if 'ETS_DEBUG' in os.environ:

            # Attempt to only skip errors in importing the backend modules.
            # The idea here is that this only happens when the last entry in
            # the traceback's stack frame mentions the toolkit in question.
            import traceback
            frames = traceback.extract_tb(sys.exc_traceback)
            filename, lineno, function, text = frames[-1]
            if not _toolkit_backend in filename:
                raise

    return be_obj



from traits.testing.unittest_tools import unittest
from traits.etsconfig.api import ETSConfig

if ETSConfig.toolkit not in ['', 'qt4']:
    raise unittest.SkipTest("TestEnamlEditor: Enaml does not support WX")

try:
    from enaml.widgets.api import Label
    from traits_enaml.testing.gui_test_assistant import GuiTestAssistant
except ImportError:
    raise unittest.SkipTest("Enaml not installed")

from traits.api import Str
from pyface.tasks.api import EnamlEditor


class DummyStrEditor(EnamlEditor):

    obj = Str

    def create_component(self):
        return Label(text=self.obj)

class TestEnamlEditor(GuiTestAssistant, unittest.TestCase):

    ###########################################################################
    # 'TestCase' interface
    ###########################################################################

    def setUp(self):
        GuiTestAssistant.setUp(self)
        self.obj = 'test message'
        self.editor = DummyStrEditor(obj=self.obj)
        with self.event_loop():
            self.editor.create(None)

    def tearDown(self):
        if self.editor.control is not None:
            with self.delete_widget(self.editor.control):
                self.editor.destroy()
        del self.editor
        GuiTestAssistant.tearDown(self)

    ###########################################################################
    # Tests
    ###########################################################################

    def test_creation(self):
        self.assertIsInstance(self.editor.component, Label)
        self.assertEqual(self.editor.component.text, self.obj)
        self.assertIsNotNone(self.editor.control)

    def test_destroy(self):
        editor = self.editor
        with self.delete_widget(editor.control):
            editor.destroy()
        self.assertIsNone(editor.control)
        # Second destruction is a no-op.
        editor.destroy()



def collapse_json(text, indent=8):
    """Compacts a string of json data by collapsing whitespace after the specified
    indent level
    
    NOTE: will not produce correct results when indent level is not a multiple
    of the json indent level
    """
    initial = " " * indent
    out = []  # final json output
    sublevel = []  # accumulation list for sublevel entries
    pending = None  # holder for consecutive entries at exact indent level
    for line in text.splitlines():
        if line.startswith(initial):
            if line[indent] == " ":
                # found a line indented further than the indent level, so add
                # it to the sublevel list
                if pending:
                    # the first item in the sublevel will be the pending item
                    # that was the previous line in the json
                    sublevel.append(pending)
                    pending = None
                item = line.strip()
                sublevel.append(item)
                if item.endswith(","):
                    sublevel.append(" ")
            elif sublevel:
                # found a line at the exact indent level *and* we have sublevel
                # items. This means the sublevel items have come to an end
                sublevel.append(line.strip())
                out.append("".join(sublevel))
                sublevel = []
            else:
                # found a line at the exact indent level but no items indented
                # further, so possibly start a new sub-level
                if pending:
                    # if there is already a pending item, it means that
                    # consecutive entries in the json had the exact same
                    # indentation and that last pending item was not the start
                    # of a new sublevel.
                    out.append(pending)
                pending = line.rstrip()
        else:
            if pending:
                # it's possible that an item will be pending but not added to
                # the output yet, so make sure it's not forgotten.
                out.append(pending)
                pending = None
            if sublevel:
                out.append("".join(sublevel))
            out.append(line)
    return "\n".join(out)


if __name__ == "__main__":
    import json
    
    s = {"zero": ["first", {"second": 2, "third": 3, "fourth": 4, "items": [[1,2,3,4], [5,6,7,8], 9, 10, [11, [12, [13, [14, 15]]]]]}]}
    
    text = json.dumps(s, indent=4)
    
    for level in range(0, 21, 4):
        processed = collapse_json(text, indent=level)
        print level
        print processed
        rebuilt = json.loads(processed)
        assert rebuilt == s



# Standard library imports.
import sys
import os

# Major package imports.
import wx
import numpy as np

# Enthought library imports.
from traits.api import on_trait_change, Any, Bool, Int, Str, List, Event, Enum, Instance, File, Unicode, Property, provides
from pyface.key_pressed_event import KeyPressedEvent

# Local imports.
from omnivore import get_image_path
from omnivore.tasks.hex_edit.hex_editor import HexEditor
from omnivore.framework.document import Document
from omnivore.arch.machine import predefined
from omnivore.utils.wx.bitviewscroller import BitmapScroller
from omnivore.utils.command import Overlay
from omnivore.utils.searchutil import HexSearcher, CharSearcher
from omnivore.utils.drawutil import get_bounds
from omnivore.utils.sortutil import invert_rects
from omnivore.tasks.hex_edit.commands import ChangeByteCommand, PasteCommand
from omnivore.framework.mouse_handler import MouseHandler

from commands import *


class MainBitmapScroller(BitmapScroller):
    """Subclass adapts the mouse interface to the MouseHandler class
    
    """
    default_mouse_handler = MouseHandler

    def __init__(self, *args, **kwargs):
        BitmapScroller.__init__(self, *args, **kwargs)

        p = get_image_path("icons/hand.ico")
        self.hand_cursor = wx.Cursor(p, wx.BITMAP_TYPE_ICO, 16, 16)
        p = get_image_path("icons/hand_closed.ico")
        self.hand_closed_cursor = wx.Cursor(p, wx.BITMAP_TYPE_ICO, 16, 16)
        self.forced_cursor = None
        self.set_mouse_mode(self.default_mouse_handler)  # dummy initial mouse handler
        self.default_pan_mode = SelectMode(self)
        self.batch = None

    def set_mouse_mode(self, handler):
        self.release_mouse()
        self.mouse_mode = handler(self)
    
    def set_cursor(self, mode=None):
        if (self.forced_cursor is not None):
            self.SetCursor(self.forced_cursor)
            #
            return

        if mode is None:
            mode = self.mouse_mode
        c = mode.get_cursor()
        self.SetCursor(c)

    def get_effective_tool_mode(self, event):
        middle_down = False
        alt_down = False
        if (event is not None):
            try:
                alt_down = event.AltDown()
                # print self.is_alt_key_down
            except:
                pass
            try:
                middle_down = event.MiddleIsDown()
            except:
                pass
        if alt_down or middle_down:
            mode = self.default_pan_mode
        else:
            mode = self.mouse_mode
        return mode

    def release_mouse(self):
        self.mouse_is_down = False
        self.selection_box_is_being_defined = False
        while self.HasCapture():
            self.ReleaseMouse()

    def on_left_down(self, evt):
        # ScrolledWindow won't take focus (and therefore won't get keyboard
        # events) unless focus explicitly set
        self.SetFocus()
        mode = self.get_effective_tool_mode(evt)
        self.forced_cursor = None
        self.selection_box_is_being_defined = False
        self.mouse_down_position = evt.GetPosition()
        self.mouse_move_position = self.mouse_down_position
        mode.num_clicks = 1
        mode.process_left_down(evt)
        self.set_cursor(mode)

    def on_motion(self, evt):
        mode = self.get_effective_tool_mode(evt)
        if evt.LeftIsDown():
            mode.process_mouse_motion_down(evt)
        else:
            mode.process_mouse_motion_up(evt)
        self.set_cursor(mode)

    def on_left_up(self, evt):
        mode = self.get_effective_tool_mode(evt)
        self.forced_cursor = None
        mode.process_left_up(evt)
        self.set_cursor(mode)

    def on_left_dclick(self, evt):
        self.SetFocus() # see comment in on_left_click
        mode = self.get_effective_tool_mode(evt)
        mode.num_clicks = 2
        mode.process_left_dclick(evt)
        self.set_cursor(mode)

    def on_popup(self, evt):
        mode = self.get_effective_tool_mode(evt)
        self.forced_cursor = None
        mode.process_popup(evt)
        self.set_cursor(mode)

    def on_mouse_wheel(self, evt):
        mode = self.get_effective_tool_mode(evt)
        mode.process_mouse_wheel(evt)
        self.set_cursor(mode)

    def on_mouse_enter(self, evt):
        self.set_cursor()

    def on_mouse_leave(self, evt):
        self.SetCursor(wx.StockCursor(wx.CURSOR_ARROW))
        self.mouse_mode.process_mouse_leave(evt)

    def on_char(self, evt):
        mode = self.get_effective_tool_mode(evt)
        self.set_cursor(mode)
        
        mode.process_key_char(evt)
    
    def on_focus(self, evt):
        mode = self.get_effective_tool_mode(evt)
        mode.process_focus(evt)
    
    def on_focus_lost(self, evt):
        mode = self.get_effective_tool_mode(evt)
        mode.process_focus_lost(evt)


class SelectMode(MouseHandler):
    icon = "select.png"
    menu_item_name = "Select"
    menu_item_tooltip = "Select regions"
    
    def display_coords(self, evt, extra=None):
        c = self.canvas
        e = c.editor
        if e is not None:
            index, bit, inside = c.event_coords_to_byte(evt)
            r0, c0 = c.byte_to_row_col(index)
            msg = "row=%d (0x%x) col=%d (0x%x) index=%d (0x%x)" % (r0, r0, c0, c0, index, index)
            if extra:
                msg += " " + extra
            e.task.status_bar.message = msg
    
    def get_display_rect(self):
        c = self.canvas
        anchor_start, anchor_end, (r1, c1), (r2, c2) = c.get_highlight_indexes()
        extra = None
        if r1 >= 0:
            w = c2 - c1
            h = r2 - r1
            if w > 0 or h > 0:
                extra = "rectangle: width=%d (0x%x), height=%d (0x%x)" % (w, w, h, h)
        return extra

    def process_left_down(self, evt):
        BitmapScroller.on_left_down(self.canvas, evt)  # can't use self.canvas directly because it has an overridded method on_left_down
        self.display_coords(evt)

    def process_left_up(self, evt):
        BitmapScroller.on_left_up(self.canvas, evt)  # can't use self.canvas directly because it has an overridded method on_left_down

    def process_mouse_motion_down(self, evt):
        self.canvas.set_selection_from_event(evt)
        self.display_coords(evt, self.get_display_rect())

    def process_mouse_motion_up(self, evt):
        self.display_coords(evt)
    
    def zoom_mouse_wheel(self, evt, amount):
        if amount < 0:
            self.canvas.zoom_out()
        elif amount > 0:
            self.canvas.zoom_in()
    
    def get_popup_actions(self, evt):
        return self.canvas.get_popup_actions()


class PickTileMode(SelectMode):
    icon = "eyedropper.png"
    menu_item_name = "Pick Tile"
    menu_item_tooltip = "Pick a tile from the map and use as the current draw tile"
    
    def init_post_hook(self):
        self.last_index = None

    def process_left_down(self, evt):
        c = self.canvas
        index, bit, inside = c.event_coords_to_byte(evt)
        if not inside:
            return
        e = c.editor
        value = e.segment[index]
        if self.last_index != index:
            e.set_cursor(index, False)
            e.character_set.set_selected_char(value)
            e.index_clicked(index, bit, None)
            e.character_set.Refresh()
        self.last_index = index
        self.display_coords(evt, "tile=%d" % value)

    def process_mouse_motion_down(self, evt):
        self.process_left_down(evt)

    def process_left_up(self, evt):
        self.last_index = None


class DrawMode(SelectMode):
    icon = "shape_freehand.png"
    menu_item_name = "Draw"
    menu_item_tooltip = "Draw with current tile"

    def draw(self, evt, start=False):
        c = self.canvas
        e = c.editor
        if e is None:
            return
        bytes = e.draw_pattern
        if not bytes:
            return
        if start:
            self.batch = DrawBatchCommand()
        byte, bit, inside = c.event_coords_to_byte(evt)
        if inside:
            e.set_cursor(byte, False)
            index = e.cursor_index
            cmd = ChangeByteCommand(e.segment, index, index+len(bytes), bytes, False, True)
            e.process_command(cmd, self.batch)

    def process_left_down(self, evt):
        self.draw(evt, True)
        self.display_coords(evt)

    def process_mouse_motion_down(self, evt):
        self.draw(evt)
        self.display_coords(evt)

    def process_left_up(self, evt):
        c = self.canvas
        e = c.editor
        if e is None:
            return
        e.end_batch()
        self.batch = None


class OverlayMode(SelectMode):
    command = None
    
    def get_display_rect(self, index):
        c = self.canvas
        i1 = self.start_index
        i2 = index
        if i2 < i1:
            i1, i2 = i2, i1
        (x1, y1), (x2, y2) = get_bounds(i1, i2, c.bytes_per_row)
        extra = None
        w = x2 - x1 + 1
        h = y2 - y1 + 1
        if w > 0 or h > 0:
            extra = "rectangle: width=%d (0x%x), height=%d (0x%x)" % (w, w, h, h)
        return extra
    
    def draw(self, evt, start=False):
        c = self.canvas
        e = c.editor
        if e is None:
            return
        bytes = e.draw_pattern
        if not bytes:
            return
        byte, bit, inside = c.event_coords_to_byte(evt)
        if inside:
            if start:
                self.batch = Overlay()
                self.start_index = byte
            e.set_cursor(byte, False)
            index = byte
            cmd = self.command(e.segment, self.start_index, index, bytes)
            e.process_command(cmd, self.batch)
            self.display_coords(evt, self.get_display_rect(index))

    def process_left_down(self, evt):
        self.draw(evt, True)

    def process_mouse_motion_down(self, evt):
        self.draw(evt)

    def process_left_up(self, evt):
        c = self.canvas
        e = c.editor
        if e is None:
            return
        e.end_batch()
        self.batch = None

class LineMode(OverlayMode):
    icon = "shape_line.png"
    menu_item_name = "Line"
    menu_item_tooltip = "Draw line with current tile"
    command = LineCommand

class SquareMode(OverlayMode):
    icon = "shape_hollow_square.png"
    menu_item_name = "Square"
    menu_item_tooltip = "Draw square with current tile"
    command = SquareCommand

class FilledSquareMode(OverlayMode):
    icon = "shape_filled_square.png"
    menu_item_name = "Filled Square"
    menu_item_tooltip = "Draw filled square with current tile"
    command = FilledSquareCommand


class BitmapEditor(HexEditor):
    """ The toolkit specific implementation of a HexEditor.  See the
    IHexEditor interface for the API documentation.
    """
    ##### class attributes
    
    valid_mouse_modes = [SelectMode, PickTileMode, DrawMode, LineMode, SquareMode, FilledSquareMode]
    
    ##### traits
    
    antic_tile_map = Any
    
    draw_pattern = Any(None)
    
    # Class attributes (not traits)
    
    rect_select = True
    
    searchers = [HexSearcher, CharSearcher]
    
    ##### Default traits
    
    def _antic_tile_map_default(self):
        return []
    
    def _map_width_default(self):
        return 40
    
    def _draw_pattern_default(self):
        return [0]

    ###########################################################################
    # 'FrameworkEditor' interface.
    ###########################################################################

    def process_extra_metadata(self, doc, e):
        HexEditor.process_extra_metadata(self, doc, e)
        pass
    
    @on_trait_change('machine.bitmap_shape_change_event,machine.bitmap_color_change_event')
    def update_bitmap(self):
        self.bitmap.recalc_view()
    
    @on_trait_change('machine.font_change_event')
    def update_fonts(self):
        pass
    
    @on_trait_change('machine.disassembler_change_event')
    def update_disassembler(self):
        pass
    
    def reconfigure_panes(self):
        self.bitmap.recalc_view()
    
    def refresh_panes(self):
        self.bitmap.refresh_view()
    
    def rebuild_document_properties(self):
        self.find_segment("Playfield map")
        self.update_mouse_mode(SelectMode)
    
    def view_segment_set_width(self, segment):
        self.bitmap_width = segment.map_width
    
    def update_mouse_mode(self, mouse_handler=None):
        if mouse_handler is not None:
            self.mouse_mode_factory = mouse_handler
        self.bitmap.set_mouse_mode(self.mouse_mode_factory)

    def set_current_draw_pattern(self, pattern, control):
        try:
            iter(pattern)
        except TypeError:
            self.draw_pattern = [pattern]
        else:
            self.draw_pattern = pattern
        if control != self.tile_map:
            self.tile_map.clear_tile_selection()
        if control != self.character_set:
            self.character_set.clear_tile_selection()

    def mark_index_range_changed(self, index_range):
        pass
    
    def perform_idle(self):
        pass
    
    def process_paste_data_object(self, data_obj, cmd_cls=None):
        bytes, extra = self.get_numpy_from_data_object(data_obj)
        ranges, indexes = self.get_selected_ranges_and_indexes()
        print extra
        if extra is None:
            cmd = PasteCommand(self.segment, ranges, self.cursor_index, indexes)
        else:
            if cmd_cls is None:
                cmd_cls = PasteRectangularCommand
            format_id, r, c = extra
            cmd = cmd_cls(self.segment, self.anchor_start_index, r, c, self.bitmap.bytes_per_row, bytes)
        self.process_command(cmd)
    
    def create_clipboard_data_object(self):
        if self.anchor_start_index != self.anchor_end_index:
            anchor_start, anchor_end, (r1, c1), (r2, c2) = self.bitmap.get_highlight_indexes()
            print anchor_start, anchor_end, (r1, c1), (r2, c2)
            bpr = self.bitmap.bytes_per_row
            last = r2 * bpr
            print last
            d = self.segment[:last].reshape(-1, bpr)
            print d
            data = d[r1:r2, c1:c2]
            print data
            data_obj = wx.CustomDataObject("numpy,columns")
            data_obj.SetData("%d,%d,%s" % (r2 - r1, c2 - c1, data.tostring()))
            return data_obj
        return None
    
    def highlight_selected_ranges(self):
        s = self.segment
        s.clear_style_bits(selected=True)
        s.set_style_ranges_rect(self.selected_ranges, selected=True)
        self.document.change_count += 1
    
    def invert_selection_ranges(self, ranges):
        rects = [(rect[2], rect[3]) for rect in [self.segment.get_rect_indexes(r[0], r[1]) for r in ranges]]
        inverted = invert_rects(rects, self.bitmap.total_rows, self.bitmap.bytes_per_row)
        ranges = self.segment.rects_to_ranges(inverted)
        return ranges
    
    def get_extra_segment_savers(self, segment):
        return []
    
    ###########################################################################
    # Trait handlers.
    ###########################################################################


    ###########################################################################
    # Private interface.
    ###########################################################################

    def _create_control(self, parent):
        """ Creates the toolkit-specific control for the widget. """

        # Base-class constructor.
        self.bitmap = MainBitmapScroller(parent, self.task)

        ##########################################
        # Events.
        ##########################################

        # Get related controls
        self.segment_list = self.window.get_dock_pane('bitmap_edit.segments').control
        self.undo_history = self.window.get_dock_pane('bitmap_edit.undo').control

        # Load the editor's contents.
        self.load()

        return self.bitmap

    #### wx event handlers ####################################################
    
    def index_clicked(self, index, bit, control):
        self.cursor_index = index
        if control != self.bitmap:
            self.bitmap.select_index(index)
        self.can_copy = (self.anchor_start_index != self.anchor_end_index)



from __future__ import absolute_import

from traits.testing.unittest_tools import unittest, UnittestTools

from ..constant import CANCEL, NO, OK, YES
from ..gui import GUI
from ..toolkit import toolkit_object
from ..window import Window

ModalDialogTester = toolkit_object('util.modal_dialog_tester:ModalDialogTester')
no_modal_dialog_tester = (ModalDialogTester.__name__ == 'Unimplemented')


class TestWindow(unittest.TestCase, UnittestTools):

    def setUp(self):
        self.gui = GUI()
        self.window = Window()

    def test_destroy(self):
        # test that destroy works even when no control
        self.window.destroy()

    def test_open_close(self):
        # test that opening and closing works as expected
        with self.assertTraitChanges(self.window, 'opening', count=1):
            with self.assertTraitChanges(self.window, 'opened', count=1):
                self.window.open()
        self.gui.process_events()
        with self.assertTraitChanges(self.window, 'closing', count=1):
            with self.assertTraitChanges(self.window, 'closed', count=1):
                self.window.close()
        self.gui.process_events()

    def test_show(self):
        # test that showing works as expected
        self.window._create()
        self.window.show(True)
        self.gui.process_events()
        self.window.show(False)
        self.gui.process_events()
        self.window.destroy()

    def test_activate(self):
        # test that activation works as expected
        self.window.open()
        self.gui.process_events()
        self.window.activate()
        self.gui.process_events()
        self.window.close()

    def test_position(self):
        # test that default position works as expected
        self.window.position = (100, 100)
        self.window.open()
        self.gui.process_events()
        self.window.close()

    def test_reposition(self):
        # test that changing position works as expected
        self.window.open()
        self.gui.process_events()
        self.window.position = (100, 100)
        self.gui.process_events()
        self.window.close()

    def test_size(self):
        # test that default size works as expected
        self.window.size = (100, 100)
        self.window.open()
        self.gui.process_events()
        self.window.close()

    def test_resize(self):
        # test that changing size works as expected
        self.window.open()
        self.gui.process_events()
        self.window.size = (100, 100)
        self.gui.process_events()
        self.window.close()

    def test_title(self):
        # test that default title works as expected
        self.window.title = "Test Title"
        self.window.open()
        self.gui.process_events()
        self.window.close()

    def test_retitle(self):
        # test that changing title works as expected
        self.window.open()
        self.gui.process_events()
        self.window.title = "Test Title"
        self.gui.process_events()
        self.window.close()

    @unittest.skipIf(no_modal_dialog_tester, 'ModalDialogTester unavailable')
    def test_confirm_reject(self):
        # test that cancel works as expected
        tester = ModalDialogTester(
            lambda: self.window.confirm("message", cancel=True))
        tester.open_and_run(when_opened=lambda x: x.close(accept=False))
        self.assertEqual(tester.result, CANCEL)

    @unittest.skipIf(no_modal_dialog_tester, 'ModalDialogTester unavailable')
    def test_confirm_yes(self):
        # test that yes works as expected
        tester = ModalDialogTester(lambda: self.window.confirm("message"))
        tester.open_and_wait(when_opened=lambda x: x.click_button(YES))
        self.gui.process_events()
        self.assertEqual(tester.result, YES)

    @unittest.skipIf(no_modal_dialog_tester, 'ModalDialogTester unavailable')
    def test_confirm_no(self):
        # test that no works as expected
        tester = ModalDialogTester(lambda: self.window.confirm("message"))
        tester.open_and_wait(when_opened=lambda x: x.click_button(NO))
        self.gui.process_events()
        self.assertEqual(tester.result, NO)

    @unittest.skipIf(no_modal_dialog_tester, 'ModalDialogTester unavailable')
    def test_confirm_cancel(self):
        # test that cncel works as expected
        tester = ModalDialogTester(
            lambda: self.window.confirm("message", cancel=True))
        tester.open_and_wait(when_opened=lambda x: x.click_button(CANCEL))
        self.gui.process_events()
        self.assertEqual(tester.result, CANCEL)

    @unittest.skipIf(no_modal_dialog_tester, 'ModalDialogTester unavailable')
    def test_information_accept(self):
        # test that information works as expected
        tester = ModalDialogTester(lambda: self.window.information("message"))
        tester.open_and_run(when_opened=lambda x: x.close(accept=True))
        self.assertIsNone(tester.result)

    @unittest.skipIf(no_modal_dialog_tester, 'ModalDialogTester unavailable')
    def test_information_ok(self):
        # test that information works as expected
        tester = ModalDialogTester(lambda: self.window.information("message"))
        tester.open_and_wait(when_opened=lambda x: x.click_button(OK))
        self.assertIsNone(tester.result)

    @unittest.skipIf(no_modal_dialog_tester, 'ModalDialogTester unavailable')
    def test_warning_accept(self):
        # test that warning works as expected
        tester = ModalDialogTester(lambda: self.window.warning("message"))
        tester.open_and_run(when_opened=lambda x: x.close(accept=True))
        self.assertIsNone(tester.result)

    @unittest.skipIf(no_modal_dialog_tester, 'ModalDialogTester unavailable')
    def test_warning_ok(self):
        # test that warning works as expected
        tester = ModalDialogTester(lambda: self.window.warning("message"))
        tester.open_and_wait(when_opened=lambda x: x.click_button(OK))
        self.assertIsNone(tester.result)

    @unittest.skipIf(no_modal_dialog_tester, 'ModalDialogTester unavailable')
    def test_error_accept(self):
        # test that error works as expected
        tester = ModalDialogTester(lambda: self.window.error("message"))
        tester.open_and_run(when_opened=lambda x: x.close(accept=True))
        self.assertIsNone(tester.result)

    @unittest.skipIf(no_modal_dialog_tester, 'ModalDialogTester unavailable')
    def test_error_ok(self):
        # test that error works as expected
        tester = ModalDialogTester(lambda: self.window.error("message"))
        tester.open_and_wait(when_opened=lambda x: x.click_button(OK))
        self.assertIsNone(tester.result)



#------------------------------------------------------------------------------
# Copyright (c) 2007, Riverbank Computing Limited
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD license.
# However, when used with the GPL version of PyQt the additional terms described in the PyQt GPL exception also apply

#
# Author: Riverbank Computing Limited
# Description: <Enthought pyface package component>
#------------------------------------------------------------------------------


# Standard library imports.
import sys

# Major package imports.
from pyface.qt import QtCore, QtGui

# Enthought library imports.
from traits.api import Bool, Event, provides, Unicode

# Local imports.
from pyface.i_python_editor import IPythonEditor, MPythonEditor
from pyface.key_pressed_event import KeyPressedEvent
from pyface.widget import Widget
from pyface.ui.qt4.code_editor.code_widget import AdvancedCodeWidget


@provides(IPythonEditor)
class PythonEditor(MPythonEditor, Widget):
    """ The toolkit specific implementation of a PythonEditor.  See the
    IPythonEditor interface for the API documentation.
    """


    #### 'IPythonEditor' interface ############################################

    dirty = Bool(False)

    path = Unicode

    show_line_numbers = Bool(True)

    #### Events ####

    changed = Event

    key_pressed = Event(KeyPressedEvent)

    ###########################################################################
    # 'object' interface.
    ###########################################################################

    def __init__(self, parent, **traits):
        super(PythonEditor, self).__init__(**traits)
        self.control = self._create_control(parent)

    ###########################################################################
    # 'PythonEditor' interface.
    ###########################################################################

    def load(self, path=None):
        """ Loads the contents of the editor.
        """
        if path is None:
            path = self.path

        # We will have no path for a new script.
        if len(path) > 0:
            f = open(self.path, 'r')
            text = f.read()
            f.close()
        else:
            text = ''

        self.control.code.setPlainText(text)
        self.dirty = False

    def save(self, path=None):
        """ Saves the contents of the editor.
        """
        if path is None:
            path = self.path

        f = open(path, 'w')
        f.write(self.control.code.toPlainText())
        f.close()

        self.dirty = False

    def select_line(self, lineno):
        """ Selects the specified line.
        """
        self.control.code.set_line_column(lineno, 0)
        self.control.code.moveCursor(QtGui.QTextCursor.EndOfLine,
                                     QtGui.QTextCursor.KeepAnchor)

    ###########################################################################
    # Trait handlers.
    ###########################################################################

    def _path_changed(self):
        self._changed_path()

    def _show_line_numbers_changed(self):
        if self.control is not None:
            self.control.code.line_number_widget.setVisible(
                self.show_line_numbers)
            self.control.code.update_line_number_width()

    ###########################################################################
    # Private interface.
    ###########################################################################

    def _create_control(self, parent):
        """ Creates the toolkit-specific control for the widget.
        """
        self.control = control = AdvancedCodeWidget(parent)
        self._show_line_numbers_changed()

        # Install event filter to trap key presses.
        event_filter = PythonEditorEventFilter(self, self.control)
        self.control.installEventFilter(event_filter)
        self.control.code.installEventFilter(event_filter)

        # Connect signals for text changes.
        control.code.modificationChanged.connect(self._on_dirty_changed)
        control.code.textChanged.connect(self._on_text_changed)

        # Load the editor's contents.
        self.load()

        return control

    def _on_dirty_changed(self, dirty):
        """ Called whenever a change is made to the dirty state of the
            document.
        """
        self.dirty = dirty

    def _on_text_changed(self):
        """ Called whenever a change is made to the text of the document.
        """
        self.changed = True


class PythonEditorEventFilter(QtCore.QObject):
    """ A thin wrapper around the advanced code widget to handle the key_pressed
        Event.
    """

    def __init__(self, editor, parent):
        super(PythonEditorEventFilter, self).__init__(parent)
        self.__editor = editor

    def eventFilter(self, obj, event):
        """ Reimplemented to trap key presses.
        """
        if self.__editor.control and obj == self.__editor.control and \
               event.type() == QtCore.QEvent.FocusOut:
            # Hack for Traits UI compatibility.
            self.__editor.control.emit(QtCore.SIGNAL('lostFocus'))

        elif self.__editor.control and obj == self.__editor.control.code and \
               event.type() == QtCore.QEvent.KeyPress:
            # Pyface doesn't seem to be Unicode aware.  Only keep the key code
            # if it corresponds to a single Latin1 character.
            kstr = event.text()
            try:
                kcode = ord(str(kstr))
            except:
                kcode = 0

            mods = event.modifiers()
            self.key_pressed = KeyPressedEvent(
                alt_down     = ((mods & QtCore.Qt.AltModifier) ==
                                QtCore.Qt.AltModifier),
                control_down = ((mods & QtCore.Qt.ControlModifier) ==
                                QtCore.Qt.ControlModifier),
                shift_down   = ((mods & QtCore.Qt.ShiftModifier) ==
                                QtCore.Qt.ShiftModifier),
                key_code     = kcode,
                event        = event)

        return super(PythonEditorEventFilter, self).eventFilter(obj, event)



""" Action definitions for HexEdit task

"""
import os
import sys

import wx
import wx.lib.dialogs

# Enthought library imports.
from traits.api import on_trait_change, Any, Int
from pyface.api import YES, NO
from pyface.action.api import Action, ActionItem
from pyface.tasks.action.api import TaskAction, EditorAction

from atrcopy import match_bit_mask, comment_bit_mask, data_bit_mask, selected_bit_mask, add_xexboot_header, add_atr_header, BootDiskImage, SegmentData, interleave_segments

from omnivore.framework.actions import *
from commands import *
from omnivore.arch.disasm import ANTIC_DISASM, JUMPMAN_LEVEL, JUMPMAN_HARVEST
from omnivore.arch.ui.antic_colors import AnticColorDialog
from omnivore.utils.wx.dialogs import prompt_for_hex, prompt_for_string, prompt_for_emulator, prompt_for_assembler, get_file_dialog_wildcard, SegmentOrderDialog, ListReorderDialog, SegmentInterleaveDialog
from omnivore.arch.machine import Machine
from omnivore.framework.minibuffer import *

class FontChoiceGroup(TaskDynamicSubmenuGroup):
    """Dynamic menu group to display the available fonts
    """
    #### 'DynamicSubmenuGroup' interface ######################################

    event_name = 'machine_menu_changed'

    ###########################################################################
    # Private interface.
    ###########################################################################

    def _get_items(self, event_data=None):
        items = []
        if event_data is not None:
            for font in event_data.font_list:
                action = UseFontAction(font=font)
                items.append(ActionItem(action=action))
            
        return items

class UseFontAction(EditorAction):
    font = Any
    
    def _name_default(self):
        return "%s" % (self.font['name'])
    
    def perform(self, event):
        self.active_editor.machine.set_font(self.font)

class LoadFontAction(EditorAction):
    name = 'Load Font...'
    
    def perform(self, event):
        dialog = FileDialog(parent=event.task.window.control)
        if dialog.open() == OK:
            self.active_editor.machine.load_font(event.task, dialog.path)

class GetFontFromSelectionAction(EditorAction):
    name = 'Get Font From Selection'
    enabled_name = 'grid_range_selected'
    
    def perform(self, event):
        self.active_editor.get_font_from_selection()


class EmulatorChoiceGroup(TaskDynamicSubmenuGroup):
    """Dynamic menu group to display the available fonts
    """
    #### 'DynamicSubmenuGroup' interface ######################################

    event_name = 'machine_menu_changed'

    ###########################################################################
    # Private interface.
    ###########################################################################

    def _get_items(self, event_data=None):
        items = []
        if event_data is not None:
            for emu in event_data.emulator_list:
                action = UseEmulatorAction(emulator=emu)
                items.append(ActionItem(action=action))
            
        return items

class UseEmulatorAction(EditorAction):
    style = 'radio'
    emulator = Any
    
    def _name_default(self):
        return "%s" % (self.emulator['name'])
    
    def perform(self, event):
        self.active_editor.document.set_emulator(self.emulator)
 
    @on_trait_change('active_editor.document')
    def _update_checked(self):
        if self.active_editor:
            self.checked = self.active_editor.document.emulator == self.emulator

class AddNewEmulatorAction(EditorAction):
    name = 'Add New Emulator...'
    
    def perform(self, event):
        emu = prompt_for_emulator(event.task.window.control, "New Emulator")
        if emu:
            self.active_editor.machine.add_emulator(event.task, emu)

class EditEmulatorsAction(EditorAction):
    name = 'Edit Emulators...'
    
    def perform(self, event):
        dlg = ListReorderDialog(event.task.window.control, Machine.get_user_defined_emulator_list(), lambda a:a['name'], prompt_for_emulator, "Manage Emulators")
        if dlg.ShowModal() == wx.ID_OK:
            emus = dlg.get_items()
            Machine.set_user_defined_emulator_list(event.task, emus)
        dlg.Destroy()

class SetSystemDefaultEmulatorAction(EditorAction):
    name = 'Set Current as System Default'
    
    def perform(self, event):
        Machine.set_system_default_emulator(event.task, self.active_editor.document.emulator)

class RunEmulatorAction(NameChangeAction):
    name = 'Run Emulator'
    accelerator = 'F5'
    menu_item_name = 'emulator_label'
    
    def perform(self, event):
        self.active_editor.run_emulator()


# Assembler syntax section

class AssemblerChoiceGroup(TaskDynamicSubmenuGroup):
    """Dynamic menu group to display the available assemblers used to display
    the disassembly syntax
    """
    #### 'DynamicSubmenuGroup' interface ######################################

    event_name = 'machine_menu_changed'

    ###########################################################################
    # Private interface.
    ###########################################################################

    def _get_items(self, event_data=None):
        items = []
        if event_data is not None:
            for asm in event_data.assembler_list:
                action = UseAssemblerAction(assembler=asm)
                items.append(ActionItem(action=action))
            
        return items

class UseAssemblerAction(EditorAction):
    style = 'radio'
    assembler = Any
    
    def _name_default(self):
        return "%s" % (self.assembler['name'])
    
    def perform(self, event):
        self.active_editor.machine.set_assembler(self.assembler)
 
    @on_trait_change('active_editor.machine.assembler')
    def _update_checked(self):
        if self.active_editor:
            self.checked = self.active_editor.machine.assembler == self.assembler

class AddNewAssemblerAction(EditorAction):
    name = 'Add New Assembler...'
    
    def perform(self, event):
        d = prompt_for_assembler(event.task.window.control, "New Assembler")
        if d:
            self.active_editor.machine.add_assembler(event.task, d)

class EditAssemblersAction(EditorAction):
    name = 'Edit Assemblers...'
    
    def perform(self, event):
        items = Machine.assembler_list
        dlg = ListReorderDialog(event.task.window.control, Machine.assembler_list, lambda a:a['name'], prompt_for_assembler, "Manage Assemblers")
        if dlg.ShowModal() == wx.ID_OK:
            asms = dlg.get_items()
            Machine.set_assembler_list(event.task, asms)
            self.active_editor.machine.verify_current_assembler()
        dlg.Destroy()

class SetSystemDefaultAssemblerAction(EditorAction):
    name = 'Set Current as System Default'
    
    def perform(self, event):
        Machine.set_system_default_assembler(event.task, self.active_editor.machine.assembler)


class FontRendererAction(EditorAction):
    """Radio buttons for changing font style
    """
    # Traits
    style = 'radio'
    
    font_renderer = Any
    
    def _name_default(self):
        return self.font_renderer.name

    def perform(self, event):
        self.active_editor.machine.set_font(font_renderer=self.font_renderer)

    @on_trait_change('active_editor.machine.font_renderer')
    def _update_checked(self):
        if self.active_editor:
            self.checked = self.active_editor.machine.font_renderer == self.font_renderer


class FontMappingAction(EditorAction):
    """Radio buttons for changing font style
    """
    # Traits
    style = 'radio'
    
    font_mapping = Any
    
    def _name_default(self):
        return self.font_mapping.name

    def perform(self, event):
        self.active_editor.machine.set_font_mapping(self.font_mapping)

    @on_trait_change('active_editor.machine.font_mapping')
    def _update_checked(self):
        if self.active_editor:
            self.checked = self.active_editor.machine.font_mapping == self.font_mapping


class BitmapRendererAction(EditorAction):
    """Radio buttons for changing font style
    """
    # Traits
    style = 'radio'
    
    bitmap_renderer = Any
    
    def _name_default(self):
        return self.bitmap_renderer.name

    def perform(self, event):
        self.active_editor.machine.set_bitmap_renderer(self.bitmap_renderer)

    @on_trait_change('active_editor.machine.bitmap_renderer')
    def _update_checked(self):
        if self.active_editor:
            self.checked = self.active_editor.machine.bitmap_renderer == self.bitmap_renderer


class BitmapWidthAction(EditorAction):
    name = "Bitmap Width"

    def perform(self, event):
        e = self.active_editor
        width = prompt_for_hex(e.window.control, 'Enter new bitmap width in bytes', 'Set Bitmap Width', e.bitmap_width)
        if width is not None and width > 0:
            wx.CallAfter(e.set_bitmap_width, width)


class BitmapZoomAction(EditorAction):
    name = "Bitmap Zoom"

    def perform(self, event):
        e = self.active_editor
        width = prompt_for_hex(e.window.control, 'Enter new pixel zoom factor', 'Set Bitmap Zoom', e.bitmap_zoom)
        if width is not None and width > 0:
            wx.CallAfter(e.set_bitmap_zoom, width)


class FontMappingWidthAction(EditorAction):
    name = "Map Width"

    def perform(self, event):
        e = self.active_editor
        width = prompt_for_hex(e.window.control, 'Enter new map width in bytes', 'Set Map Width', str(e.map_width))
        if width is not None and width > 0:
            wx.CallAfter(e.set_map_width, width)


class FontMappingZoomAction(EditorAction):
    name = "Map Zoom"

    def perform(self, event):
        e = self.active_editor
        width = prompt_for_hex(e.window.control, 'Enter new pixel zoom factor', 'Set Map Zoom', e.map_zoom)
        if width is not None and width > 0:
            wx.CallAfter(e.set_map_zoom, width)


class AnticColorAction(EditorAction):
    name = 'Choose Colors...'
    
    def perform(self, event):
        e = self.active_editor
        dlg = AnticColorDialog(event.task.window.control, e.machine.antic_color_registers)
        if dlg.ShowModal() == wx.ID_OK:
            e.machine.update_colors(dlg.colors)


class UseColorsAction(EditorAction):
    name = 'Use Colors'
    colors = Any
    
    def perform(self, event):
        self.active_editor.machine.update_colors(self.colors)


class ColorStandardAction(EditorAction):
    style = 'radio'
    
    color_standard = Int

    def perform(self, event):
        self.active_editor.machine.set_color_standard(self.color_standard)

    @on_trait_change('active_editor.machine.color_standard')
    def _update_checked(self):
        if self.active_editor:
            self.checked = self.active_editor.machine.color_standard == self.color_standard


class TextFontAction(EditorAction):
    name = 'Text Font...'
    
    def perform(self, event):
        e = self.active_editor
        data = wx.FontData()
        data.EnableEffects(True)
        data.SetColour(e.machine.text_color)
        data.SetInitialFont(e.machine.text_font)
        dlg = wx.FontDialog(self.active_editor.control, data)
        if dlg.ShowModal() == wx.ID_OK:
            data = dlg.GetFontData()
            e.machine.set_text_font(e, data.GetChosenFont(), data.GetColour())
            e.reconfigure_panes()


class PredefinedMachineAction(EditorAction):
    """Radio buttons for changing font style
    """
    # Traits
    style = 'radio'
    
    machine = Any
    
    def _name_default(self):
        return self.machine.name

    def perform(self, event):
        self.active_editor.set_machine(self.machine)

    @on_trait_change('active_editor.machine')
    def _update_checked(self):
        if self.active_editor:
            self.checked = self.active_editor.machine == self.machine


class ProcessorTypeAction(EditorAction):
    """Radio buttons for changing the processor type and therefore disassembler
    """
    # Traits
    style = 'radio'
    
    disassembler = Any
    
    def _name_default(self):
        return self.disassembler.name

    def perform(self, event):
        self.active_editor.machine.set_disassembler(self.disassembler)

    @on_trait_change('active_editor.machine.disassembler')
    def _update_checked(self):
        if self.active_editor:
            self.checked = self.active_editor.machine.disassembler == self.disassembler


class MemoryMapAction(EditorAction):
    """Radio buttons for changing the memory map
    """
    # Traits
    style = 'radio'
    
    memory_map = Any
    
    def _name_default(self):
        return self.memory_map.name

    def perform(self, event):
        self.active_editor.machine.set_memory_map(self.memory_map)

    @on_trait_change('active_editor.machine.memory_map')
    def _update_checked(self):
        if self.active_editor:
            self.checked = self.active_editor.machine.memory_map == self.memory_map


class CurrentSegmentParserAction(NameChangeAction):
    name = '<current parser>'
    enabled = False
    enabled_name = None
    menu_item_name = 'segment_parser_label'

    def perform(self, event):
        pass

    def _enabled_update(self):
        # override the lookup of the enabled_name trait and simply force it to
        # be disabled
        self.enabled = False

class SegmentParserAction(EditorAction):
    """Radio buttons for changing font style
    """
    # Traits
    style = 'toggle'
    
    segment_parser = Any
    
    def _name_default(self):
        return self.segment_parser.menu_name

    def perform(self, event):
        self.active_editor.set_segment_parser(self.segment_parser)

    @on_trait_change('task.segments_changed')
    def _update_checked(self):
        if self.active_editor:
            new_state = self.active_editor.document.segment_parser.__class__ == self.segment_parser
            # workaround to force reset of state because sometimes the toggle
            # is not turned off otherwise
            self.checked = not new_state
            self.checked = new_state

class SegmentChoiceGroup(TaskDynamicSubmenuGroup):
    """Dynamic menu group to display the available fonts
    """
    #### 'DynamicSubmenuGroup' interface ######################################

    event_name = 'segments_changed'

    ###########################################################################
    # Private interface.
    ###########################################################################

    def _get_items(self, event_data=None):
        items = []
        if event_data is not None:
            for i, segment in enumerate(event_data):
                if sys.platform == "darwin":
                    action = UseSegmentAction(segment=segment, segment_number=i, task=self.task)
                else:
                    action = UseSegmentRadioAction(segment=segment, segment_number=i, task=self.task, checked=False)
                log.debug("SegmentChoiceGroup: created %s for %s, num=%d" % (action, str(segment), i))
                items.append(ActionItem(action=action, parent=self))
            
        return items

class UseSegmentAction(EditorAction):
    segment = Any
    
    segment_number = Int
    
    def _name_default(self):
        return str(self.segment)
    
    def perform(self, event):
        self.active_editor.view_segment_number(self.segment_number)

class UseSegmentRadioAction(UseSegmentAction):
    style = 'radio'

    @on_trait_change('task.segment_selected')
    def _update_checked(self):
        if self.active_editor:
            state = self.active_editor.segment_number == self.segment_number
            log.debug("UseSegmentAction: checked=%s %s %s %s" % (state, str(self.segment), self.active_editor.segment_number, self.segment_number))
            self.checked = state


class SelectSegmentInAllAction(EditorAction):
    name = 'Select This Segment in All'
    
    segment_number = Int
    
    def perform(self, event):
        e = self.active_editor
        s = e.document.segments[self.segment_number]
        e.view_segment_number(0)
        e.select_none(False)
        s.set_style_ranges([(0, len(s))], selected=True)
        e.adjust_selection(s)
        e.index_clicked(e.anchor_start_index, 0, None)

class GetSegmentFromSelectionAction(EditorAction):
    name = 'New Segment From Selection'
    enabled_name = 'can_copy'
    
    def perform(self, event):
        e = self.active_editor
        text = prompt_for_string(e.window.control, "Enter segment name", "New Segment")
        if text is not None:
            segment, = e.get_segments_from_selection()
            if not text:
                text = "%04x-%04x" % (segment.start_addr, segment.start_addr + len(segment) - 1)
            segment.name = text
            e.add_user_segment(segment)


class MultipleSegmentsFromSelectionAction(EditorAction):
    name = 'Multiple Segments From Selection'
    enabled_name = 'can_copy'
    
    def perform(self, event):
        e = self.active_editor
        size = prompt_for_hex(e.window.control, "Enter number of bytes in each segment", "Multiple Segments")
        if size is not None and size > 0:
            segments = e.get_segments_from_selection(size)
            for segment in segments:
                text = "%04x-%04x" % (segment.start_addr, segment.start_addr + len(segment) - 1)
                segment.name = text
                e.add_user_segment(segment, False)
            e.update_segments_ui()


class InterleaveSegmentsAction(EditorAction):
    name = 'Interleave Segments'
    tooltip = 'Create new segment by interleaving segments'
    accelerator = 'F12'

    def get_bytes(self, dlg):
        return dlg.get_bytes()

    def perform(self, event):
        e = self.active_editor
        dlg = SegmentInterleaveDialog(e.window.control, "Interleave Segments", e.document.segments[1:])
        if dlg.ShowModal() == wx.ID_OK:
            s = dlg.get_segments()
            print s
            factor = dlg.get_interleave()
            print factor
            segment = interleave_segments(s, factor)
            e.add_user_segment(segment, False)
            e.update_segments_ui()
        dlg.Destroy()


class MarkSelectionAsCodeAction(EditorAction):
    name = 'Mark Selection As Code'
    enabled_name = 'can_copy'
    
    def perform(self, event):
        e = self.active_editor
        s = e.segment
        ranges = s.get_style_ranges(selected=True)
        s.clear_style_ranges(ranges, data=True, user=1)
        e.document.change_count += 1
        e.metadata_dirty = True
        e.mark_index_range_changed(ranges[0])
        e.refresh_panes()


class MarkSelectionAsDataAction(EditorAction):
    name = 'Mark Selection As Data'
    enabled_name = 'can_copy'
    
    def perform(self, event):
        e = self.active_editor
        s = e.segment
        ranges = s.get_style_ranges(selected=True)
        s.clear_style_ranges(ranges, user=1)
        s.set_style_ranges(ranges, data=True)
        e.document.change_count += 1
        e.metadata_dirty = True
        # check if the segment can be merged with a previous data segment
        index = ranges[0][0]
        while index > 0 and s.style[index-1] & data_bit_mask:
            index -= 1
        e.mark_index_range_changed((index, ranges[0][1]))
        e.refresh_panes()


class CustomDisassemblerAction(EditorAction):
    name = 'Mark Selection As <custom>'
    enabled_name = 'can_copy'
    disassembly_type = 0

    def perform(self, event):
        e = self.active_editor
        s = e.segment
        ranges = s.get_style_ranges(selected=True)
        s.set_style_ranges(ranges, data=True, user=1)
        s.set_user_data(ranges, 1, self.disassembly_type)
        e.document.change_count += 1
        e.metadata_dirty = True
        e.mark_index_range_changed(ranges[0])
        e.refresh_panes()

class MarkSelectionAsDisplayListAction(CustomDisassemblerAction):
    name = 'Mark Selection As Display List'
    disassembly_type = ANTIC_DISASM

class MarkSelectionAsJumpmanLevelAction(CustomDisassemblerAction):
    name = 'Mark Selection As Jumpman Level Data'
    enabled_name = 'can_copy'
    disassembly_type = JUMPMAN_LEVEL

class MarkSelectionAsJumpmanHarvestAction(CustomDisassemblerAction):
    name = 'Mark Selection As Jumpman Harvest Table'
    enabled_name = 'can_copy'
    disassembly_type = JUMPMAN_HARVEST


class CopyDisassemblyAction(EditorAction):
    name = 'Copy Disassembly'
    enabled_name = 'can_copy'
    
    def perform(self, event):
        e = self.active_editor
        s = e.segment
        ranges = s.get_style_ranges(selected=True)
        lines = []
        try:
            for start, end in ranges:
                lines.extend(e.disassembly.get_disassembled_text(start, end))
        except IndexError:
            e.window.error("Disassembly still in progress...")
            return
        text = os.linesep.join(lines) + os.linesep
        data_obj = wx.TextDataObject()
        data_obj.SetText(text)
        e.set_clipboard_object(data_obj)


class CopyAsReprAction(EditorAction):
    name = 'Copy as Escaped String'
    enabled_name = 'can_copy'
    
    def perform(self, event):
        e = self.active_editor
        ranges, indexes = e.get_selected_ranges_and_indexes()
        data = e.segment[indexes]
        s1 = repr(data.tostring())
        q = s1[0]
        text = s1[1:-1]  # remove leading/trailing quotes
        if q == "'":
            # double quotes are literal, single quotes are escaped
            text = text.replace('"', "\\x22").replace("\\'", "\\x27")
        else:
            # single quotes are literal, double are escaped
            text = text.replace("'", "\\x27").replace('\\"', "\\x22")
        data_obj = wx.TextDataObject()
        data_obj.SetText(text)
        e.set_clipboard_object(data_obj)


def prompt_for_comment(e, s, ranges, desc):
    existing = s.get_first_comment(ranges)
    text = prompt_for_string(e.window.control, desc, "Add Comment", existing)
    if text is not None:
        s.set_comment(ranges, text)
        e.document.change_count += 1
        e.metadata_dirty = True
        e.refresh_panes()
    
class AddCommentAction(EditorAction):
    name = 'Add Comment'
    accelerator = 'Alt+C'
    
    def perform(self, event):
        e = self.active_editor
        s = e.segment
        if e.can_copy:
            ranges = s.get_style_ranges(selected=True)
            desc = "Enter comment for range %s" % str(ranges)
        else:
            index = e.cursor_index
            ranges = [(index, index+1)]
            desc = "Enter comment for location %s" % index
        prompt_for_comment(e, s, ranges, desc)

class AddCommentPopupAction(EditorAction):
    name = 'Add Comment'
    
    def perform(self, event):
        e = self.active_editor
        s = e.segment
        if event.popup_data["in_selection"]:
            ranges = s.get_style_ranges(selected=True)
            desc = "Enter comment for range %s" % e.get_label_of_ranges(ranges)
        else:
            index = event.popup_data["index"]
            ranges = [(index, index+1)]
            desc = "Enter comment for location %s" % e.get_label_at_index(index)
        if ranges:
            prompt_for_comment(e, s, ranges, desc)
    
class RemoveCommentAction(EditorAction):
    name = 'Remove Comment'
    accelerator = 'Ctrl+Alt+C'
    
    def perform(self, event):
        e = self.active_editor
        s = e.segment
        if e.can_copy:
            ranges = s.get_style_ranges(selected=True)
        else:
            index = e.cursor_index
            ranges = [(index, index+1)]
        if ranges:
            s.clear_comment(ranges)
            e.document.change_count += 1
            e.metadata_dirty = True
            e.refresh_panes()

class RemoveCommentPopupAction(EditorAction):
    name = 'Remove Comment'
    
    def perform(self, event):
        e = self.active_editor
        s = e.segment
        if event.popup_data["in_selection"]:
            ranges = s.get_style_ranges(selected=True)
        else:
            index = event.popup_data["index"]
            ranges = [(index, index+1)]
        if ranges:
            s.clear_comment(ranges)
            e.document.change_count += 1
            e.metadata_dirty = True
            e.refresh_panes()


class DeleteUserSegmentAction(EditorAction):
    name = 'Delete User Segment'
    segment_number = Int
    
    def perform(self, event):
        e = self.task.active_editor
        segment = e.document.segments[self.segment_number]
        if self.task.confirm("Delete user segment %s?" % segment.name, "Delete User Segment") == YES:
            e.delete_user_segment(segment)


class SetSegmentOriginAction(EditorAction):
    name = 'Set Segment Origin'
    segment_number = Int
    
    def perform(self, event):
        e = self.active_editor
        segment = e.document.segments[self.segment_number]
        org = prompt_for_hex(e.window.control, "Enter origin address for %s" % segment.name, "Set Segment Origin")
        if org is not None:
            segment.start_addr = org
            e.update_segments_ui()
            e.metadata_dirty = True
            e.reconfigure_panes()


class SaveAsXEXAction(EditorAction):
    name = 'Export as XEX...'
    tooltip = 'Create executable from segments'
    title = 'Create Executable'
    file_ext = "xex"

    def get_bytes(self, dlg):
        return dlg.get_bytes()

    def get_dialog(self, e):
        return SegmentOrderDialog(e.window.control, self.title, e.document.segments[1:])

    def perform(self, event):
        e = self.active_editor
        dlg = self.get_dialog(e)
        if dlg.ShowModal() == wx.ID_OK:
            bytes = self.get_bytes(dlg)
            dialog = FileDialog(default_filename="test.%s" % self.file_ext, parent=e.window.control, action='save as')
            if dialog.open() == OK:
                self.active_editor.save_to_uri(bytes, dialog.path, False)
        dlg.Destroy()


class SaveAsXEXBootAction(SaveAsXEXAction):
    name = 'Export as Boot Disk...'
    tooltip = 'Create a bootable disk from segments'
    title = 'Create Boot Disk'
    file_ext = "atr"

    def get_bytes(self, dlg):
        xex = dlg.get_bytes()
        title, author = dlg.get_extra_text()[0:2]
        print title, author
        bytes = add_xexboot_header(xex, title=title, author=author)
        bytes = add_atr_header(bytes)
        rawdata = SegmentData(bytes)
        atr = BootDiskImage(rawdata)
        print atr.header
        return atr.bytes.tostring()

    def get_dialog(self, e):
        return SegmentOrderDialog(e.window.control, self.title, e.document.segments[1:], "Segment Order for Boot Disk", True)


class SaveSegmentAsFormatAction(EditorAction):
    saver = Any
    
    segment_number = Int
    
    def _name_default(self):
        return "%s (%s)" % (self.saver.export_data_name, self.saver.export_extensions[0])
    
    def perform(self, event):
        segment = self.task.active_editor.document.segments[self.segment_number]
        dialog = FileDialog(default_filename=segment.name, parent=event.task.window.control, action='save as', wildcard=get_file_dialog_wildcard(self.saver.export_data_name, self.saver.export_extensions))
        if dialog.open() == OK:
            self.active_editor.save_segment(self.saver, dialog.path)

class SaveSegmentGroup(TaskDynamicSubmenuGroup):
    """ A menu for changing the active task in a task window.
    """
    id = 'SaveSegmentGroup'
    
    event_name = 'segment_selected'

    def _get_items(self, event_data=None):
        items = []
        if event_data is not None:
            segment_number = event_data
            segment = self.task.active_editor.document.segments[segment_number]
            savers = self.task.active_editor.get_extra_segment_savers(segment)
            savers.extend(segment.savers)
            for saver in savers:
                action = SaveSegmentAsFormatAction(saver=saver, segment_number=segment_number)
                items.append(ActionItem(action=action))
        return items


class GotoIndexAction(Action):
    addr_index = Int()
    
    segment_num = Int()
    
    def perform(self, event):
        e = self.active_editor
        if self.segment_num >= 0:
            e.view_segment_number(self.segment_num)
        e.index_clicked(self.addr_index, 0, None)


class SegmentGotoAction(EditorAction):
    name = "Goto Address..."
    accelerator = 'Alt+G'

    def perform(self, event):
        e = self.active_editor
        addr, error = prompt_for_hex(e.window.control, "Enter address value: (default hex; prefix with # for decimal)", "Goto Address in a Segment", return_error=True, default_base="hex")
        if addr is not None:
            s = e.segment
            index = addr - s.start_addr
            if e.segment.is_valid_index(index):
                e.index_clicked(index, 0, None)
                e.task.status_bar.message = e.get_label_at_index(index)
            else:
                segment_num, segment, index = e.document.find_segment_in_range(addr)
                if segment_num >= 0:
                    e.view_segment_number(segment_num)
                    e.index_clicked(index, 0, None)
                    e.task.status_bar.message = "%s in segment %s" % (e.get_label_at_index(index), e.segment.name)
                else:
                    e.task.status_bar.message = "Address $%04x not valid in any segment" % addr
        else:
            e.task.status_bar.message = error

class InsertFileAction(EditorAction):
    name = 'Insert File...'
    
    def perform(self, event):
        dialog = FileDialog(parent=event.task.window.control)
        if dialog.open() == OK:
            e = self.active_editor
            cmd = InsertFileCommand(e.segment, e.cursor_index, dialog.path)
            e.process_command(cmd)


class IndexRangeAction(EditorAction):
    enabled_name = 'can_copy'
    cmd = None
    
    def _name_default(self):
        return self.cmd.pretty_name

    def get_cmd(self, editor, segment, ranges):
        return self.cmd(segment, ranges)
    
    def perform(self, event):
        e = self.active_editor
        ranges = e.get_optimized_selected_ranges()
        cmd = self.get_cmd(e, e.segment, ranges)
        self.active_editor.process_command(cmd)

class ZeroAction(IndexRangeAction):
    cmd = ZeroCommand
    accelerator = 'Ctrl+0'

class FFAction(IndexRangeAction):
    cmd = FFCommand
    accelerator = 'Ctrl+9'

class NOPAction(IndexRangeAction):
    cmd = NOPCommand
    accelerator = 'Ctrl+3'

    def get_cmd(self, editor, segment, ranges):
        nop = editor.machine.get_nop()
        return self.cmd(segment, ranges, nop)

class SetHighBitAction(IndexRangeAction):
    cmd = SetHighBitCommand

class ClearHighBitAction(IndexRangeAction):
    cmd = ClearHighBitCommand

class BitwiseNotAction(IndexRangeAction):
    cmd = BitwiseNotCommand
    accelerator = 'Ctrl+1'

class LeftShiftAction(IndexRangeAction):
    cmd = LeftShiftCommand

class RightShiftAction(IndexRangeAction):
    cmd = RightShiftCommand

class LeftRotateAction(IndexRangeAction):
    cmd = LeftRotateCommand
    accelerator = 'Ctrl+<'

class RightRotateAction(IndexRangeAction):
    cmd = RightRotateCommand
    accelerator = 'Ctrl+>'

class ReverseBitsAction(IndexRangeAction):
    cmd = ReverseBitsCommand


class IndexRangeValueAction(IndexRangeAction):
    def _name_default(self):
        return self.cmd.pretty_name + "..."
    
    def show_dialog(self, e):
        value = prompt_for_hex(e.window.control, "Enter byte value: (prefix with 0x or $ for hex, % for binary)", self.cmd.pretty_name)
        if value is not None:
            cmd = self.cmd(e.segment, e.selected_ranges, value)
            self.active_editor.process_command(cmd)
            
    def perform(self, event):
        wx.CallAfter(self.show_dialog, self.active_editor)

class SetValueAction(IndexRangeValueAction):
    cmd = SetValueCommand

class OrWithAction(IndexRangeValueAction):
    cmd = OrWithCommand
    accelerator = 'Ctrl+\\'

class AndWithAction(IndexRangeValueAction):
    cmd = AndWithCommand
    accelerator = 'Ctrl+7'

class XorWithAction(IndexRangeValueAction):
    cmd = XorWithCommand
    accelerator = 'Ctrl+6'

class RampUpAction(IndexRangeValueAction):
    cmd = RampUpCommand

class RampDownAction(IndexRangeValueAction):
    cmd = RampDownCommand

class AddValueAction(IndexRangeValueAction):
    cmd = AddValueCommand
    accelerator = 'Ctrl+='

class SubtractValueAction(IndexRangeValueAction):
    cmd = SubtractValueCommand
    accelerator = 'Ctrl+-'

class SubtractFromAction(IndexRangeValueAction):
    cmd = SubtractFromCommand
    accelerator = 'Shift+Ctrl+-'

class MultiplyAction(IndexRangeValueAction):
    cmd = MultiplyCommand
    accelerator = 'Ctrl+8'

class DivideByAction(IndexRangeValueAction):
    cmd = DivideByCommand
    accelerator = 'Ctrl+/'

class DivideFromAction(IndexRangeValueAction):
    cmd = DivideFromCommand
    accelerator = 'Shift+Ctrl+/'


class PasteAndRepeatAction(EditorAction):
    name = 'Paste and Repeat'
    accelerator = 'Shift+Ctrl+V'
    tooltip = 'Paste and repeat clipboard data until current selection is filled'
    enabled_name = 'can_copy'
    
    def perform(self, event):
        self.active_editor.paste(PasteAndRepeatCommand)


class FindAction(EditorAction):
    name = 'Find'
    accelerator = 'Ctrl+F'
    tooltip = 'Find bytes or characters in the raw data or in disassembly comments'

    def perform(self, event):
        e = self.active_editor
        event.task.show_minibuffer(NextPrevTextMinibuffer(e, FindAllCommand, FindNextCommand, FindPrevCommand, initial=e.last_search_settings["find"]))

class FindNextAction(EditorAction):
    name = 'Find Next'
    accelerator = 'Ctrl+G'
    tooltip = 'Find next match'

    def perform(self, event):
        e = self.active_editor
        event.task.show_minibuffer(NextPrevTextMinibuffer(e, FindAllCommand, FindNextCommand, FindPrevCommand, next_match=True, initial=e.last_search_settings["find"]))

class FindPrevAction(EditorAction):
    name = 'Find Previous'
    accelerator = 'Shift+Ctrl+G'
    tooltip = 'Find previous match'

    def perform(self, event):
        e = self.active_editor
        event.task.show_minibuffer(NextPrevTextMinibuffer(e, FindAllCommand, FindNextCommand, FindPrevCommand, prev_match=True, initial=e.last_search_settings["find"]))

class FindAlgorithmAction(EditorAction):
    name = 'Find Using Expression'
    accelerator = 'Alt+Ctrl+F'
    tooltip = 'Find bytes using logical and arithmetic comparisons'

    def perform(self, event):
        e = self.active_editor
        event.task.show_minibuffer(NextPrevTextMinibuffer(e, FindAlgorithmCommand, FindNextCommand, FindPrevCommand, initial=e.last_search_settings["algorithm"], help_text=" Use variable 'a' for address, 'b' for byte values. (Mouse over for examples)", help_tip="Examples:\n\nAll bytes after the 10th byte: a > 10\n\nBytes with values > 128 but only after the 10th byte: (b > 128) & (a > 10)\n\n"))

class FindToSelectionAction(EditorAction):
    name = 'Find to Selection'
    accelerator = 'Alt+Ctrl+A'
    tooltip = 'Convert all matched locations to multi-selection'

    def perform(self, event):
        e = self.active_editor
        e.convert_ranges({'match':True}, {'selected':True})
        e.refresh_panes()
        event.task.on_hide_minibuffer_or_cancel(None)

class CancelMinibufferAction(EditorAction):
    name = 'Cancel Minibuffer or current edit'
    accelerator = 'ESC'
    tooltip = 'Remove minibuffer or cancel current edit'

    def perform(self, event):
        event.task.on_hide_minibuffer_or_cancel(None)

class ViewDiffHighlightAction(EditorAction):
    name = 'Show Baseline Differences'
    tooltip = 'Show bytes that are different than the baseline version'
    style = 'toggle'
    enabled_name = 'baseline_present'

    def perform(self, event):
        e = self.active_editor
        value = not e.diff_highlight
        e.diff_highlight = value
        print value
        if value:
            e.compare_to_baseline()
        else:
            e.document.clear_baseline()
        e.refresh_panes()

    @on_trait_change('active_editor.diff_highlight')
    def _update_checked(self):
        if self.active_editor:
            self.checked = self.active_editor.diff_highlight

class LoadBaselineVersionAction(EditorAction):
    name = 'Add Baseline Difference File...'
    tooltip = 'Add baseline file to be used to show differences in current version'
    
    def perform(self, event):
        dialog = FileDialog(parent=event.task.window.control)
        if dialog.open() == OK:
            e = self.active_editor
            e.load_baseline(dialog.path)
            e.compare_to_baseline()
            e.refresh_panes()

class RevertToBaselineAction(IndexRangeAction):
    cmd = RevertToBaselineCommand
    enabled_name = 'can_copy_baseline'

class FindNextBaselineDiffAction(EditorAction):
    name = 'Find Next Difference'
    accelerator = 'Ctrl+D'
    tooltip = 'Find next difference to the baseline'
    enabled_name = 'diff_highlight'

    def perform(self, event):
        e = self.active_editor
        index = e.cursor_index
        new_index = e.segment.find_next(index, diff=True)
        if new_index is not None:
            e.index_clicked(new_index, 0, None)

class FindPrevBaselineDiffAction(EditorAction):
    name = 'Find Previous Difference'
    accelerator = 'Ctrl+Shift+D'
    tooltip = 'Find previous difference to the baseline'
    enabled_name = 'diff_highlight'

    def perform(self, event):
        e = self.active_editor
        index = e.cursor_index
        new_index = e.segment.find_previous(index, diff=True)
        if new_index is not None:
            e.index_clicked(new_index, 0, None)

class ListDiffAction(EditorAction):
    name = 'List Differences'
    tooltip = 'Show a text representation of the differences'
    enabled_name = 'diff_highlight'

    def perform(self, event):
        e = self.active_editor
        s = e.segment
        ranges = s.get_style_ranges(diff=True)
        d = e.machine.get_disassembler(e.task.hex_grid_lower_case, e.task.assembly_lower_case)
        d.set_pc(s, s.start_addr)
        lines = []
        blank_label = ""
        for start, end in ranges:
            bytes = list(s[start:end])
            origin = s.label(start, e.task.hex_grid_lower_case)
            comment = s.get_comment(start)
            if comment:
                lines.append("%-8s; %s" % (blank_label, comment))
            lines.append("%-8s%s $%s" % (blank_label, d.asm_origin, origin))
            lines.append("%-8s%s" % (blank_label, d.get_data_byte_string(bytes)))
            lines.append("")
        text = "\n".join(lines) + "\n"
        dlg = wx.lib.dialogs.ScrolledMessageDialog(e.task.window.control, text, "Summary of Differences")
        dlg.ShowModal()

class UndoCursorPositionAction(EditorAction):
    name = 'Previous Cursor Position'
    accelerator = 'Ctrl+-'
    tooltip = 'Go to previous cursor position in cursor history'

    def perform(self, event):
        e = self.active_editor
        e.undo_cursor_history()

class RedoCursorPositionAction(EditorAction):
    name = 'Next Cursor Position'
    accelerator = 'Shift+Ctrl+-'
    tooltip = 'Go to next cursor position in cursor history'

    def perform(self, event):
        e = self.active_editor
        e.redo_cursor_history()



from traits.api import HasTraits, provides

from omnivore.file_type.i_file_recognizer import IFileRecognizer
from omnivore.utils.textutil import guessBinary
from omnivore.utils.permute import PermutePrivate
from omnivore.framework.document import Document

@provides(IFileRecognizer)
class PlainTextRecognizer(HasTraits):
    """Default plain text identifier based on percentage of non-ASCII bytes.
    
    """
    id = "text/plain"
    
    def identify(self, guess):
        if not guessBinary(guess.get_utf8()):
            return "text/plain"

@provides(IFileRecognizer)
class XMLTextRecognizer(HasTraits):
    """Default plain text identifier based on percentage of non-ASCII bytes.
    
    """
    id = "text/xml"
    
    before = "text/plain"
    
    def identify(self, guess):
        byte_stream = guess.get_utf8().strip()
        if not byte_stream.startswith("<"):
            return
        if byte_stream.startswith("<?xml"):
            found_xml = True
        else:
            found_xml = False
        if "<rss" in byte_stream:
            return "application/rss+xml"
        byte_stream = byte_stream.lower()
        if "<!doctype html" in byte_stream or "<html" in byte_stream:
            return "text/html"
        
        if found_xml:
            return "text/xml"

@provides(IFileRecognizer)
class PoundBangTextRecognizer(HasTraits):
    """Recognizer for pound bang style executable script files
    
    """
    id = "text/poundbang"
    
    before = "text/plain"
    
    def identify(self, guess):
        byte_stream = guess.get_utf8()
        if not byte_stream.startswith("#!"):
            return
        line = byte_stream[2:80].lower().strip()
        if line.startswith("/usr/bin/env"):
            line = line[12:].strip()
        words = line.split()
        names = words[0].split("/")
        if names[-1]:
            return "text/%s" % names[-1]

@provides(IFileRecognizer)
class PrivateTextRecognizer(HasTraits):
    """Recognizer for pound bang style executable script files
    
    """
    id = "text/private"
    
    before = "text/poundbang"

    header = "#!omnivore-private\n"
    
    def identify(self, guess):
        byte_stream = guess.get_utf8()
        if not byte_stream.startswith(self.header):
            return
        return self.id
    
    def load(self, guess):
        start = len(self.header)
        doc = Document(metadata=guess.metadata, bytes=guess.numpy[start:])
        doc.permute = PermutePrivate()
        print "loading %s: %s" % (self.id, doc)
        return doc



import numpy as np

atascii_to_internal = np.hstack([np.arange(64, 96, dtype=np.uint8),np.arange(64, dtype=np.uint8),np.arange(96, 128, dtype=np.uint8)])
atascii_to_internal = np.hstack([atascii_to_internal, atascii_to_internal + 128])

internal_to_atascii = np.hstack([np.arange(32, 96, dtype=np.uint8),np.arange(32, dtype=np.uint8),np.arange(96, 128, dtype=np.uint8)])
internal_to_atascii = np.hstack([internal_to_atascii, internal_to_atascii + 128])



from traits.api import HasTraits, provides

from omnivore.file_type.i_file_recognizer import IFileRecognizer


@provides(IFileRecognizer)
class OmnivoreRecognizer(HasTraits):
    """Meta recognizer that forces the loading of a disk image when the
    .omnivore file is selected instead
    """
    
    name = "Omnivore Extra Metadata File"
    
    id = "text/vnd.omnivore.extra_metadata"
    
    before = "application/vnd.*"
    
    def identify(self, guess):
        uri = guess.metadata.uri
        if uri.endswith(".omnivore"):
            b = guess.bytes
            if b.startswith("#") or b.startswith("{"):
                uri = uri[0:-9]
                guess.reload(uri)



#------------------------------------------------------------------------------
#
#  Copyright (c) 2006, Enthought, Inc.
#  All rights reserved.
#
#  This software is provided without warranty under the terms of the BSD
#  license included in enthought/LICENSE.txt and may be redistributed only
#  under the conditions described in the aforementioned license.  The license
#  is also available online at http://www.enthought.com/licenses/BSD.txt
#
#  Thanks for using Enthought open source!
#
#  Author: David C. Morrill
#  Date:   06/25/2006
#
#------------------------------------------------------------------------------

""" Defines the various editors for a drag-and-drop editor,
    for the wxPython user interface toolkit. A drag-and-drop editor represents
    its value as a simple image which, depending upon the editor style, can be
    a drag source only, a drop target only, or both a drag source and a drop
    target.
"""

#-------------------------------------------------------------------------------
#  Imports:
#-------------------------------------------------------------------------------

import wx
import numpy

from cPickle \
    import load

from traits.api \
    import Bool

# FIXME: ToolkitEditorFactory is a proxy class defined here just for backward
# compatibility. The class has been moved to the
# traitsui.editors.dnd_editor file.
from traitsui.editors.dnd_editor \
    import ToolkitEditorFactory

from pyface.wx.drag_and_drop \
    import PythonDropSource, PythonDropTarget, clipboard

try:
    from apptools.io import File
except ImportError:
    File = None

try:
    from apptools.naming.api import Binding
except ImportError:
    Binding = None

from pyface.image_resource \
    import ImageResource

from editor \
    import Editor

#-------------------------------------------------------------------------------
#  Constants:
#-------------------------------------------------------------------------------

# The image to use when the editor accepts files:
file_image = ImageResource( 'file' ).create_image()

# The image to use when the editor accepts objects:
object_image = ImageResource( 'object' ).create_image()

# The image to use when the editor is disabled:
inactive_image = ImageResource( 'inactive' ).create_image()

# String types:
string_type = ( str, unicode )

#-------------------------------------------------------------------------------
#  'SimpleEditor' class:
#-------------------------------------------------------------------------------

class SimpleEditor ( Editor ):
    """ Simply style of editor for a drag-and-drop editor, which is both a drag
        source and a drop target.
    """

    #---------------------------------------------------------------------------
    #  Trait definitions:
    #---------------------------------------------------------------------------

    # Is the editor a drop target?
    drop_target = Bool( True )

    # Is the editor a drag source?
    drag_source = Bool( True )

    #---------------------------------------------------------------------------
    #  Finishes initializing the editor by creating the underlying toolkit
    #  widget:
    #---------------------------------------------------------------------------

    def init ( self, parent ):
        """ Finishes initializing the editor by creating the underlying toolkit
            widget.
        """
        # Determine the drag/drop type:
        value         = self.value
        self._is_list = isinstance( value, list )
        self._is_file = (isinstance( value, string_type ) or
                         (self._is_list and (len( value ) > 0) and
                          isinstance( value[0], string_type )))

        # Get the right image to use:
        image = self.factory.image
        if image is not None:
            image = image.create_image()
            disabled_image = self.factory.disabled_image
            if disabled_image is not None:
                disabled_image = disabled_image.create_image()
        else:
            disabled_image = inactive_image
            image          = object_image
            if self._is_file:
                image = file_image

        self._image = image.ConvertToBitmap()
        if disabled_image is not None:
            self._disabled_image = disabled_image.ConvertToBitmap()
        else:
            data = numpy.reshape( numpy.fromstring( image.GetData(),
                                                    numpy.uint8 ),
                      ( -1, 3 ) ) * numpy.array( [ [ 0.297, 0.589, 0.114 ] ] )
            g = data[ :, 0 ] + data[ :, 1 ] + data[ :, 2 ]
            data[ :, 0 ] = data[ :, 1 ] = data[ :, 2 ] = g
            image.SetData( numpy.ravel( data.astype(numpy.uint8) ).tostring() )
            image.SetMaskColour( 0, 0, 0 )
            self._disabled_image = image.ConvertToBitmap()

        # Create the control and set up the event handlers:
        self.control = control = wx.Window( parent, -1,
                         size = wx.Size( image.GetWidth(), image.GetHeight() ) )
        self.set_tooltip()

        if self.drop_target:
            control.SetDropTarget( PythonDropTarget( self ) )

        wx.EVT_LEFT_DOWN( control, self._left_down )
        wx.EVT_LEFT_UP(   control, self._left_up )
        wx.EVT_MOTION(    control, self._mouse_move )
        wx.EVT_PAINT(     control, self._on_paint )

    #---------------------------------------------------------------------------
    #  Disposes of the contents of an editor:
    #---------------------------------------------------------------------------

    def dispose ( self ):
        """ Disposes of the contents of an editor.
        """
        control = self.control
        wx.EVT_LEFT_DOWN( control, None )
        wx.EVT_LEFT_UP(   control, None )
        wx.EVT_MOTION(    control, None )
        wx.EVT_PAINT(     control, None )

        super( SimpleEditor, self ).dispose()

    #---------------------------------------------------------------------------
    #  Updates the editor when the object trait changes external to the editor:
    #---------------------------------------------------------------------------

    def update_editor ( self ):
        """ Updates the editor when the object trait changes externally to the
            editor.
        """
        return

#-- Private Methods ------------------------------------------------------------

    #---------------------------------------------------------------------------
    #  Returns the processed version of a drag request's data:
    #---------------------------------------------------------------------------

    def _get_drag_data ( self, data ):
        """ Returns the processed version of a drag request's data.
        """
        if isinstance( data, list ):

            if Binding is not None and isinstance( data[0], Binding ):
                data = [ item.obj for item in data ]

            if File is not None and isinstance( data[0], File ):
                data = [ item.absolute_path for item in data ]
                if not self._is_file:
                    result = []
                    for file in data:
                        item = self._unpickle( file )
                        if item is not None:
                            result.append( item )
                    data = result

        else:
            if Binding is not None and isinstance( data, Binding ):
                data = data.obj

            if File is not None and isinstance( data, File ):
                data = data.absolute_path
                if not self._is_file:
                    object = self._unpickle( data )
                    if object is not None:
                        data = object

        return data

    #---------------------------------------------------------------------------
    #  Returns the unpickled version of a specified file (if possible):
    #---------------------------------------------------------------------------

    def _unpickle ( self, file_name ):
        """ Returns the unpickled version of a specified file (if possible).
        """
        fh = None
        try:
            fh     = file( file_name, 'rb' )
            object = load( fh )
        except:
            object = None

        if fh is not None:
            fh.close()

        return object

#-- wxPython Event Handlers ----------------------------------------------------

    def _on_paint ( self, event ):
        """ Called when the control needs repainting.
        """
        image   = self._image
        control = self.control
        if not control.IsEnabled():
            image = self._disabled_image

        wdx, wdy = control.GetClientSizeTuple()
        wx.PaintDC( control ).DrawBitmap( image,
            (wdx - image.GetWidth())  / 2, (wdy - image.GetHeight()) / 2, True )

    def _left_down ( self, event ):
        """ Handles the left mouse button being pressed.
        """
        if self.control.IsEnabled() and self.drag_source:
            self._x, self._y = event.GetX(), event.GetY()
            self.control.CaptureMouse()

        event.Skip()

    def _left_up ( self, event ):
        """ Handles the left mouse button being released.
        """
        if self._x is not None:
            self._x = None
            self.control.ReleaseMouse()

        event.Skip()

    def _mouse_move ( self, event ):
        """ Handles the mouse being moved.
        """
        if self._x is not None:
            if ((abs( self._x - event.GetX() ) +
                 abs( self._y - event.GetY() )) >= 3):
                self.control.ReleaseMouse()
                self._x = None
                if self._is_file:
                    FileDropSource(   self.control, self.value )
                else:
                    PythonDropSource( self.control, self.value )

        event.Skip()

#----- Drag and drop event handlers: -------------------------------------------

    #---------------------------------------------------------------------------
    #  Handles a Python object being dropped on the control:
    #---------------------------------------------------------------------------

    def wx_dropped_on ( self, x, y, data, drag_result ):
        """ Handles a Python object being dropped on the tree.
        """
        try:
            self.value = self._get_drag_data( data )
            return drag_result
        except:
            return wx.DragNone

    #---------------------------------------------------------------------------
    #  Handles a Python object being dragged over the control:
    #---------------------------------------------------------------------------

    def wx_drag_over ( self, x, y, data, drag_result ):
        """ Handles a Python object being dragged over the tree.
        """
        try:
            self.object.base_trait( self.name ).validate( self.object,
                                        self.name, self._get_drag_data( data ) )
            return drag_result
        except:
            return wx.DragNone

#-------------------------------------------------------------------------------
#  'CustomEditor' class:
#-------------------------------------------------------------------------------

class CustomEditor ( SimpleEditor ):
    """ Custom style of drag-and-drop editor, which is not a drag source.
    """
    #---------------------------------------------------------------------------
    #  Trait definitions:
    #---------------------------------------------------------------------------

    # Is the editor a drag source? This value overrides the default.
    drag_source = False

#-------------------------------------------------------------------------------
#  'ReadonlyEditor' class:
#-------------------------------------------------------------------------------

class ReadonlyEditor ( SimpleEditor ):
    """ Read-only style of drag-and-drop editor, which is not a drop target.
    """
    #---------------------------------------------------------------------------
    #  Trait definitions:
    #---------------------------------------------------------------------------

    # Is the editor a drop target? This value overrides the default.
    drop_target = False

#-------------------------------------------------------------------------------
#  'FileDropSource' class:
#-------------------------------------------------------------------------------

class FileDropSource ( wx.DropSource ):
    """ Represents a draggable file.
    """
    #---------------------------------------------------------------------------
    #  Initializes the object:
    #---------------------------------------------------------------------------

    def __init__ ( self, source, files ):
        """ Initializes the object.
        """
        self.handler    = None
        self.allow_move = True

        # Put the data to be dragged on the clipboard:
        clipboard.data        = files
        clipboard.source      = source
        clipboard.drop_source = self

        data_object = wx.FileDataObject()
        if isinstance( files, string_type ):
            files = [ files ]

        for file in files:
            data_object.AddFile( file )

        # Create the drop source and begin the drag and drop operation:
        super( FileDropSource, self ).__init__( source )
        self.SetData( data_object )
        self.result = self.DoDragDrop( True )

    #---------------------------------------------------------------------------
    #  Called when the data has been dropped:
    #---------------------------------------------------------------------------

    def on_dropped ( self, drag_result ):
        """ Called when the data has been dropped. """
        return

## EOF ########################################################################
